@@ -1958,6 +1958,12 @@ ev_loop_new (unsigned int flags)
 
 #endif /* multiplicity */
 
+int
+ev_loop_refcount (EV_P)
+{
+  return activecnt;
+}
+
 #if EV_VERIFY
 static void noinline
 verify_watcher (EV_P_ W w)
