@@ -1,13 +1,17 @@
 
+from __future__ import print_function
+
 import os
+import sys
 import signal
 import threading
 import pyuv
 
+if sys.version_info >= (3, 0):
+    LINESEP = os.linesep.encode()
+else:
+    LINESEP = os.linesep
 
-def on_client_shutdown(client, error):
-    client.close()
-    clients.remove(client)
 
 def on_read(client, data, error):
     if data is None:
@@ -17,8 +21,7 @@ def on_read(client, data, error):
     data = data.strip()
     if not data:
         return
-    client.write(data+os.linesep)
-    client.shutdown(on_client_shutdown)
+    client.write(data+LINESEP)
 
 def on_connection(server, error):
     client = pyuv.TCP(server.loop)
@@ -36,7 +39,7 @@ def signal_cb(sig, frame):
     async.send(async_exit)
 
 
-print "PyUV version %s" % pyuv.__version__
+print("PyUV version %s" % pyuv.__version__)
 
 loop = pyuv.Loop()
 
@@ -58,6 +61,6 @@ signal.pause()
 
 t.join()
 
-print "Stopped!"
+print("Stopped!")
 
 
