@@ -12,32 +12,33 @@ static void
 host_cb(void *arg, int status, int timeouts, struct hostent *hostent)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    char ip[INET6_ADDRSTRLEN];
+    char **ptr;
+    ares_cb_data_t *data;
+    DNSResolver *self;
+    PyObject *callback, *dns_name, *dns_status, *dns_timeouts, *dns_aliases, *dns_result, *tmp, *result;
+
     ASSERT(arg);
 
-    ares_cb_data_t *data = (ares_cb_data_t*)arg;
-    DNSResolver *self = data->resolver;
-    PyObject *callback = data->cb;
+    data = (ares_cb_data_t*)arg;
+    self = data->resolver;
+    callback = data->cb;
+
     ASSERT(self);
     /* Object could go out of scope in the callback, increase refcount to avoid it */
     Py_INCREF(self);
 
-    char ip4[INET_ADDRSTRLEN];
-    char ip6[INET6_ADDRSTRLEN];
-    char **ptr;
-
-    PyObject *dns_name;
     if (hostent != NULL) {
         dns_name = PyString_FromString(hostent->h_name);
     } else {
         Py_INCREF(Py_None);
         dns_name = Py_None;
     }
-    PyObject *dns_status = PyLong_FromLong((long) status);
-    PyObject *dns_timeouts = PyLong_FromLong((long) timeouts);
-    PyObject *dns_aliases = PyList_New(0);
-    PyObject *dns_result = PyList_New(0);
-    PyObject *tmp;
-    PyObject *result;
+
+    dns_status = PyLong_FromLong((long) status);
+    dns_timeouts = PyLong_FromLong((long) timeouts);
+    dns_aliases = PyList_New(0);
+    dns_result = PyList_New(0);
 
     if (!(dns_status && dns_timeouts && dns_name && dns_aliases && dns_result)) {
         PyErr_NoMemory();
@@ -58,11 +59,11 @@ host_cb(void *arg, int status, int timeouts, struct hostent *hostent)
         }
         for (ptr = hostent->h_addr_list; *ptr != NULL; ptr++) {
             if (hostent->h_addrtype == AF_INET) {
-                inet_ntop(AF_INET, *ptr, ip4, INET_ADDRSTRLEN);
-                tmp = PyString_FromString(ip4);
+                uv_inet_ntop(AF_INET, *ptr, ip, INET_ADDRSTRLEN);
+                tmp = PyString_FromString(ip);
             } else {
-                inet_ntop(AF_INET6, *ptr, ip6, INET6_ADDRSTRLEN);
-                tmp = PyString_FromString(ip6);
+                uv_inet_ntop(AF_INET6, *ptr, ip, INET6_ADDRSTRLEN);
+                tmp = PyString_FromString(ip);
             }
             if (tmp == NULL) {
                 break;
@@ -92,25 +93,30 @@ static void
 nameinfo_cb(void *arg, int status, int timeouts, char *node, char *service)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    ares_cb_data_t *data;
+    DNSResolver *self;
+    PyObject *callback, *dns_status, *dns_timeouts, *dns_node, *dns_service, *result;
+
     ASSERT(arg);
 
-    ares_cb_data_t *data = (ares_cb_data_t*)arg;
-    DNSResolver *self = data->resolver;
-    PyObject *callback = data->cb;
+    data = (ares_cb_data_t*)arg;
+    self = data->resolver;
+    callback = data->cb;
+
     ASSERT(self);
     /* Object could go out of scope in the callback, increase refcount to avoid it */
     Py_INCREF(self);
 
-    PyObject *dns_status = PyLong_FromLong((long) status);
-    PyObject *dns_timeouts = PyLong_FromLong((long) timeouts);
-    PyObject *dns_node;
+    dns_status = PyLong_FromLong((long) status);
+    dns_timeouts = PyLong_FromLong((long) timeouts);
+
     if (node) {
         dns_node = PyString_FromString(node);
     } else {
         Py_INCREF(Py_None);
         dns_node = Py_None;
     }
-    PyObject *dns_service;
+
     if (service) {
         dns_service = PyString_FromString(service);
     } else {
@@ -124,7 +130,7 @@ nameinfo_cb(void *arg, int status, int timeouts, char *node, char *service)
         goto nameinfo_end;
     }
 
-    PyObject *result = PyObject_CallFunctionObjArgs(callback, self, dns_status, dns_timeouts, dns_node, dns_service, NULL);
+    result = PyObject_CallFunctionObjArgs(callback, self, dns_status, dns_timeouts, dns_node, dns_service, NULL);
     if (result == NULL) {
         PyErr_WriteUnraisable(callback);
     }
@@ -146,9 +152,7 @@ makesockaddr(struct sockaddr *addr, int addrlen)
 {
     struct sockaddr_in *addr4;
     struct sockaddr_in6 *addr6;
-    char ip4[INET_ADDRSTRLEN];
-    char ip6[INET6_ADDRSTRLEN];
-    int r = 0;
+    char ip[INET6_ADDRSTRLEN];
 
     if (addrlen == 0) {
         /* No address */
@@ -159,17 +163,15 @@ makesockaddr(struct sockaddr *addr, int addrlen)
     case AF_INET:
     {
         addr4 = (struct sockaddr_in*)addr;
-        r = uv_ip4_name(addr4, ip4, INET_ADDRSTRLEN);
-        ASSERT(r == 0);
-        return Py_BuildValue("si", ip4, ntohs(addr4->sin_port));
+        uv_ip4_name(addr4, ip, INET_ADDRSTRLEN);
+        return Py_BuildValue("si", ip, ntohs(addr4->sin_port));
     }
 
     case AF_INET6:
     {
         addr6 = (struct sockaddr_in6*)addr;
-        r = uv_ip6_name(addr6, ip6, INET6_ADDRSTRLEN);
-        ASSERT(r == 0);
-        return Py_BuildValue("siii", ip6, ntohs(addr6->sin6_port), addr6->sin6_flowinfo, addr6->sin6_scope_id);
+        uv_ip6_name(addr6, ip, INET6_ADDRSTRLEN);
+        return Py_BuildValue("siii", ip, ntohs(addr6->sin6_port), addr6->sin6_flowinfo, addr6->sin6_scope_id);
     }
 
     default:
@@ -182,22 +184,25 @@ makesockaddr(struct sockaddr *addr, int addrlen)
 static void
 getaddrinfo_cb(uv_getaddrinfo_t* handle, int status, struct addrinfo* res)
 {
-    struct addrinfo *ptr;
-    PyObject *addr;
-    PyObject *item;
-
     PyGILState_STATE gstate = PyGILState_Ensure();
+    struct addrinfo *ptr;
+    ares_cb_data_t *data;
+    DNSResolver *self;
+    PyObject *callback, *addr, *item, *dns_status, *dns_result, *result;
+
     ASSERT(handle);
 
-    ares_cb_data_t *data = (ares_cb_data_t*)(handle->data);
-    DNSResolver *self = data->resolver;
-    PyObject *callback = data->cb;
+    data = (ares_cb_data_t*)(handle->data);
+    self = data->resolver;
+    callback = data->cb;
+
     ASSERT(self);
     /* Object could go out of scope in the callback, increase refcount to avoid it */
     Py_INCREF(self);
 
-    PyObject *dns_status = PyInt_FromLong(status);
-    PyObject *dns_result = PyList_New(0);
+    dns_status = PyInt_FromLong(status);
+    dns_result = PyList_New(0);
+
     if (!(dns_status && dns_result)) {
         PyErr_NoMemory();
         PyErr_WriteUnraisable(callback);
@@ -226,7 +231,7 @@ getaddrinfo_cb(uv_getaddrinfo_t* handle, int status, struct addrinfo* res)
         Py_DECREF(item);
     }
 
-    PyObject *result = PyObject_CallFunctionObjArgs(callback, self, dns_status, dns_result, NULL);
+    result = PyObject_CallFunctionObjArgs(callback, self, dns_status, dns_result, NULL);
     if (result == NULL) {
         PyErr_WriteUnraisable(callback);
     }
@@ -236,7 +241,6 @@ getaddrinfo_end:
 
     Py_DECREF(callback);
     uv_freeaddrinfo(res);
-    handle->data = NULL;
     PyMem_Free(handle);
     PyMem_Free(data);
 
@@ -248,10 +252,10 @@ getaddrinfo_end:
 static PyObject *
 DNSResolver_func_gethostbyname(DNSResolver *self, PyObject *args, PyObject *kwargs)
 {
-    PyObject *callback;
     char *name;
     int family = AF_INET;
     ares_cb_data_t *cb_data;
+    PyObject *callback;
 
     static char *kwlist[] = {"callback", "name", "family", NULL};
 
@@ -282,14 +286,13 @@ DNSResolver_func_gethostbyname(DNSResolver *self, PyObject *args, PyObject *kwar
 static PyObject *
 DNSResolver_func_gethostbyaddr(DNSResolver *self, PyObject *args, PyObject *kwargs)
 {
-    PyObject *callback;
     char *name;
-    ares_cb_data_t *cb_data;
+    int family, length;
+    void *address;
     struct in_addr addr4;
     struct in6_addr addr6;
-    int family;
-    int length;
-    void *address;
+    ares_cb_data_t *cb_data;
+    PyObject *callback;
 
     static char *kwlist[] = {"callback", "name", NULL};
 
@@ -302,11 +305,11 @@ DNSResolver_func_gethostbyaddr(DNSResolver *self, PyObject *args, PyObject *kwar
         return NULL;
     }
 
-    if (inet_pton(AF_INET, name, &addr4) == 1) {
+    if (uv_inet_pton(AF_INET, name, &addr4) == 1) {
         family = AF_INET;
         length = sizeof(struct in_addr);
         address = (void *)&addr4;
-    } else if (inet_pton(AF_INET6, name, &addr6) == 1) {
+    } else if (uv_inet_pton(AF_INET6, name, &addr6) == 1) {
         family = AF_INET6;
         length = sizeof(struct in6_addr);
         address = (void *)&addr6;
@@ -333,20 +336,20 @@ DNSResolver_func_gethostbyaddr(DNSResolver *self, PyObject *args, PyObject *kwar
 static PyObject *
 DNSResolver_func_getnameinfo(DNSResolver *self, PyObject *args, PyObject *kwargs)
 {
-    PyObject *callback;
-    ares_cb_data_t *cb_data;
+    char *name;
+    int port, flags, length;
     struct in_addr addr4;
     struct in6_addr addr6;
     struct sockaddr *sa;
     struct sockaddr_in sa4;
     struct sockaddr_in6 sa6;
-    char *name;
-    int port = 0;
-    int flags = 0;
-    int length;
+    ares_cb_data_t *cb_data;
+    PyObject *callback;
 
     static char *kwlist[] = {"callback", "name", "port", "flags", NULL};
 
+    port = flags = 0;
+
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "Os|ii:getnameinfo", kwlist, &callback, &name, &port, &flags)) {
         return NULL;
     }
@@ -361,11 +364,11 @@ DNSResolver_func_getnameinfo(DNSResolver *self, PyObject *args, PyObject *kwargs
         return NULL;
     }
 
-    if (inet_pton(AF_INET, name, &addr4) == 1) {
+    if (uv_inet_pton(AF_INET, name, &addr4) == 1) {
         sa4 = uv_ip4_addr(name, port);
         sa = (struct sockaddr *)&sa4;
         length = sizeof(struct sockaddr_in);
-    } else if (inet_pton(AF_INET6, name, &addr6) == 1) {
+    } else if (uv_inet_pton(AF_INET6, name, &addr6) == 1) {
         sa6 = uv_ip6_addr(name, port);
         sa = (struct sockaddr *)&sa6;
         length = sizeof(struct sockaddr_in6);
@@ -391,19 +394,19 @@ DNSResolver_func_getnameinfo(DNSResolver *self, PyObject *args, PyObject *kwargs
 static PyObject *
 DNSResolver_func_getaddrinfo(DNSResolver *self, PyObject *args, PyObject *kwargs)
 {
-    PyObject *callback;
     char *name;
     char port_str[6];
-    int port, family, socktype, protocol, flags;
+    int port, family, socktype, protocol, flags, r;
     struct addrinfo hints;
     ares_cb_data_t *cb_data = NULL;
     uv_getaddrinfo_t* handle = NULL;
+    PyObject *callback;
+
+    static char *kwlist[] = {"callback", "name", "port", "family", "socktype", "protocol", "flags", NULL};
 
     port = socktype = protocol = flags = 0;
     family = AF_UNSPEC;
 
-    static char *kwlist[] = {"callback", "name", "port", "family", "socktype", "protocol", "flags", NULL};
-
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "Os|iiiii:getaddrinfo", kwlist, &callback, &name, &port, &family, &socktype, &protocol, &flags)) {
         return NULL;
     }
@@ -442,7 +445,7 @@ DNSResolver_func_getaddrinfo(DNSResolver *self, PyObject *args, PyObject *kwargs
     hints.ai_protocol = protocol;
     hints.ai_flags = flags;
 
-    int r = uv_getaddrinfo(UV_LOOP(self), handle, &getaddrinfo_cb, name, port_str, &hints);
+    r = uv_getaddrinfo(UV_LOOP(self), handle, &getaddrinfo_cb, name, port_str, &hints);
     if (r != 0) {
         raise_uv_exception(self->loop, PyExc_DNSError);
         goto error;
@@ -465,15 +468,14 @@ error:
 static int
 set_dns_servers(DNSResolver *self, PyObject *value)
 {
-    struct ares_addr_node *servers;
     char *server;
-    int i;
-    int length;
-    int ret = 0;
-
+    int i, r, length, ret;
+    struct ares_addr_node *servers;
     PyObject *server_list = value;
     PyObject *item;
 
+    ret = 0;
+
     if (!PyList_Check(server_list)) {
         PyErr_SetString(PyExc_TypeError, "servers argument must be a list");
 	return -1;
@@ -495,9 +497,9 @@ set_dns_servers(DNSResolver *self, PyObject *value)
             goto servers_set_end;
         }
 
-        if (inet_pton(AF_INET, server, &servers[i].addr.addr4) == 1) {
+        if (uv_inet_pton(AF_INET, server, &servers[i].addr.addr4) == 1) {
             servers[i].family = AF_INET;
-        } else if (inet_pton(AF_INET6, server, &servers[i].addr.addr6) == 1) {
+        } else if (uv_inet_pton(AF_INET6, server, &servers[i].addr.addr6) == 1) {
             servers[i].family = AF_INET6;
         } else {
             PyErr_SetString(PyExc_ValueError, "invalid IP address");
@@ -516,7 +518,7 @@ set_dns_servers(DNSResolver *self, PyObject *value)
         servers = NULL;
     }
 
-    int r = ares_set_servers(self->channel, servers);
+    r = ares_set_servers(self->channel, servers);
     if (r != 0) {
         PyErr_SetString(PyExc_DNSError, "error c-ares library options");
         ret = -1;
@@ -532,21 +534,21 @@ servers_set_end:
 static PyObject *
 DNSResolver_servers_get(DNSResolver *self, void *closure)
 {
-    char ip4[INET_ADDRSTRLEN];
-    char ip6[INET6_ADDRSTRLEN];
-    struct ares_addr_node *server;
-    struct ares_addr_node *servers;
-
+    int r;
+    char ip[INET6_ADDRSTRLEN];
+    struct ares_addr_node *server, *servers;
     PyObject *server_list;
     PyObject *tmp;
 
+    UNUSED_ARG(closure);
+
     server_list = PyList_New(0);
     if (!server_list) {
         PyErr_NoMemory();
         return NULL;
     }
 
-    int r = ares_get_servers(self->channel, &servers);
+    r = ares_get_servers(self->channel, &servers);
     if (r != 0) {
         PyErr_SetString(PyExc_DNSError, "error getting c-ares nameservers");
         return NULL;
@@ -554,11 +556,11 @@ DNSResolver_servers_get(DNSResolver *self, void *closure)
 
     for (server = servers; server != NULL; server = server->next) {
         if (server->family == AF_INET) {
-            inet_ntop(AF_INET, &(server->addr.addr4), ip4, INET_ADDRSTRLEN);
-            tmp = PyString_FromString(ip4);
+            uv_inet_ntop(AF_INET, &(server->addr.addr4), ip, INET_ADDRSTRLEN);
+            tmp = PyString_FromString(ip);
         } else {
-            inet_ntop(AF_INET6, &(server->addr.addr6), ip6, INET6_ADDRSTRLEN);
-            tmp = PyString_FromString(ip6);
+            uv_inet_ntop(AF_INET6, &(server->addr.addr6), ip, INET6_ADDRSTRLEN);
+            tmp = PyString_FromString(ip);
         }
         if (tmp == NULL) {
             break;
@@ -577,6 +579,7 @@ DNSResolver_servers_get(DNSResolver *self, void *closure)
 static int
 DNSResolver_servers_set(DNSResolver *self, PyObject *value, void *closure)
 {
+    UNUSED_ARG(closure);
     return set_dns_servers(self, value);
 }
 
@@ -584,6 +587,8 @@ DNSResolver_servers_set(DNSResolver *self, PyObject *value, void *closure)
 static int
 DNSResolver_tp_init(DNSResolver *self, PyObject *args, PyObject *kwargs)
 {
+    int r, optmask;
+    struct ares_options options;
     Loop *loop;
     PyObject *servers = NULL;
 
@@ -601,15 +606,12 @@ DNSResolver_tp_init(DNSResolver *self, PyObject *args, PyObject *kwargs)
     Py_INCREF(loop);
     self->loop = loop;
 
-    int r = ares_library_init(ARES_LIB_INIT_ALL);
+    r = ares_library_init(ARES_LIB_INIT_ALL);
     if (r != 0) {
         PyErr_SetString(PyExc_DNSError, "error initializing c-ares library");
         return -1;
     }
 
-    struct ares_options options;
-    int optmask;
-
     optmask = ARES_OPT_FLAGS;
     options.flags = ARES_FLAG_USEVC;
 
@@ -733,12 +735,26 @@ static PyTypeObject DNSResolverType = {
     DNSResolver_tp_new,                                             /*tp_new*/
 };
 
+#ifdef PYUV_PYTHON3
+static PyModuleDef pyuv_dns_module = {
+    PyModuleDef_HEAD_INIT,
+    "pyuv.dns",             /*m_name*/
+    NULL,                   /*m_doc*/
+    -1,                     /*m_size*/
+    NULL,                   /*m_methods*/
+};
+#endif
 
 PyObject *
 init_dns(void)
 {
     PyObject *module;
+#ifdef PYUV_PYTHON3
+    module = PyModule_Create(&pyuv_dns_module);
+#else
     module = Py_InitModule("pyuv.dns", NULL);
+#endif
+
     if (module == NULL) {
         return NULL;
     }
@@ -759,6 +775,8 @@ init_dns(void)
     PyModule_AddIntMacro(module, ARES_NI_IDN_ALLOW_UNASSIGNED);
     PyModule_AddIntMacro(module, ARES_NI_IDN_USE_STD3_ASCII_RULES);
 
+    PyUVModule_AddType(module, "DNSResolver", &DNSResolverType);
+
     return module;
 }
 
