@@ -4,7 +4,11 @@
 static void
 inscode(PyObject *module_dict, PyObject *other_dict, const char *name, int code)
 {
+#ifdef PYUV_PYTHON3
+    PyObject *error_name = PyUnicode_FromString(name);
+#else
     PyObject *error_name = PyString_FromString(name);
+#endif
     PyObject *error_code = PyInt_FromLong((long) code);
 
     /* Don't bother checking for errors; they'll be caught at the end
@@ -20,6 +24,40 @@ inscode(PyObject *module_dict, PyObject *other_dict, const char *name, int code)
 }
 
 
+static PyObject *
+Errno_func_strerror(PyObject *obj, PyObject *args)
+{
+    int errorno;
+    uv_err_t err;
+
+    UNUSED_ARG(obj);
+
+    if (!PyArg_ParseTuple(args, "i:strerror", &errorno)) {
+        return NULL;
+    }
+
+    err.code = errorno;
+    return PyString_FromString(uv_strerror(err));
+}
+
+
+static PyMethodDef
+Errno_methods[] = {
+    { "strerror", (PyCFunction)Errno_func_strerror, METH_VARARGS, "Get string representation of an error code." },
+    { NULL }
+};
+
+
+#ifdef PYUV_PYTHON3
+static PyModuleDef pyuv_errorno_module = {
+    PyModuleDef_HEAD_INIT,
+    "pyuv.errno",           /*m_name*/
+    NULL,                   /*m_doc*/
+    -1,                     /*m_size*/
+    Errno_methods,          /*m_methods*/
+};
+#endif
+
 PyObject *
 init_errno(void)
 {
@@ -27,8 +65,11 @@ init_errno(void)
     PyObject *module_dict;
     PyObject *errorcode_dict;
     PyObject *ares_errorcode_dict;
-
-    module = Py_InitModule("pyuv.errno", NULL);
+#ifdef PYUV_PYTHON3
+    module = PyModule_Create(&pyuv_errorno_module);
+#else
+    module = Py_InitModule("pyuv.errno", Errno_methods);
+#endif
     if (module == NULL) {
         return NULL;
     }
