@@ -12,7 +12,11 @@ typedef struct {
 static PyObject *
 format_time(time_t sec, unsigned long nsec)
 {
+    /* TODO: look at stat_float_times in Modules/posixmodule.c */
     PyObject *ival;
+
+    UNUSED_ARG(nsec);
+
 #if SIZEOF_TIME_T > SIZEOF_LONG
     ival = PyLong_FromLongLong((PY_LONG_LONG)sec);
 #else
@@ -28,17 +32,23 @@ format_time(time_t sec, unsigned long nsec)
 static void
 stat_cb(uv_fs_t* req) {
     PyGILState_STATE gstate = PyGILState_Ensure();
-    ASSERT(req);
-    ASSERT(req->fs_type == UV_FS_STAT || req->fs_type == UV_FS_LSTAT || req->fs_type == UV_FS_FSTAT);
-
-    struct stat *st = (struct stat *)(req->ptr);
-    fs_req_data_t *req_data = (fs_req_data_t*)(req->data);
-
+    struct stat *st;
+    fs_req_data_t *req_data;
     unsigned long ansec, mnsec, cnsec;
     PyObject *result, *errorno, *stat_data, *path;
 
+    ASSERT(req);
+    ASSERT(req->fs_type == UV_FS_STAT || req->fs_type == UV_FS_LSTAT || req->fs_type == UV_FS_FSTAT);
+
+    st = (struct stat *)(req->ptr);
+    req_data = (fs_req_data_t*)(req->data);
+
     if (req->path != NULL) {
+#ifdef PYUV_PYTHON3
+        path = PyUnicode_FromString(req->path);
+#else
         path = PyString_FromString(req->path);
+#endif
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -124,12 +134,11 @@ stat_cb(uv_fs_t* req) {
 
 end:
 
+    uv_fs_req_cleanup(req);
     Py_DECREF(req_data->loop);
     Py_DECREF(req_data->callback);
     Py_DECREF(req_data->data);
     PyMem_Free(req_data);
-    req->data = NULL;
-    uv_fs_req_cleanup(req);
     PyMem_Free(req);
 
     PyGILState_Release(gstate);
@@ -139,15 +148,20 @@ end:
 static void
 unlink_cb(uv_fs_t* req) {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    fs_req_data_t *req_data;
+    PyObject *result, *errorno, *path;
+
     ASSERT(req);
     ASSERT(req->fs_type == UV_FS_UNLINK);
 
-    fs_req_data_t *req_data = (fs_req_data_t*)(req->data);
-
-    PyObject *result, *errorno, *path;
+    req_data = (fs_req_data_t*)(req->data);
 
     if (req->path != NULL) {
+#ifdef PYUV_PYTHON3
+        path = PyUnicode_FromString(req->path);
+#else
         path = PyString_FromString(req->path);
+#endif  
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -180,15 +194,20 @@ unlink_cb(uv_fs_t* req) {
 static void
 mkdir_cb(uv_fs_t* req) {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    fs_req_data_t *req_data;
+    PyObject *result, *errorno, *path;
+
     ASSERT(req);
     ASSERT(req->fs_type == UV_FS_MKDIR);
 
-    fs_req_data_t *req_data = (fs_req_data_t*)(req->data);
-
-    PyObject *result, *errorno, *path;
+    req_data = (fs_req_data_t*)(req->data);
 
     if (req->path != NULL) {
+#ifdef PYUV_PYTHON3
+        path = PyUnicode_FromString(req->path);
+#else
         path = PyString_FromString(req->path);
+#endif
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -221,15 +240,20 @@ mkdir_cb(uv_fs_t* req) {
 static void
 rmdir_cb(uv_fs_t* req) {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    fs_req_data_t *req_data;
+    PyObject *result, *errorno, *path;
+
     ASSERT(req);
     ASSERT(req->fs_type == UV_FS_RMDIR);
 
-    fs_req_data_t *req_data = (fs_req_data_t*)(req->data);
-
-    PyObject *result, *errorno, *path;
+    req_data = (fs_req_data_t*)(req->data);
 
     if (req->path != NULL) {
+#ifdef PYUV_PYTHON3
+        path = PyUnicode_FromString(req->path);
+#else
         path = PyString_FromString(req->path);
+#endif
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -262,15 +286,21 @@ rmdir_cb(uv_fs_t* req) {
 static void
 rename_cb(uv_fs_t* req) {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    fs_req_data_t *req_data;
+    PyObject *result, *errorno, *path;
+
     ASSERT(req);
     ASSERT(req->fs_type == UV_FS_RENAME);
 
-    fs_req_data_t *req_data = (fs_req_data_t*)(req->data);
-
-    PyObject *result, *errorno, *path;
+    req_data = (fs_req_data_t*)(req->data);
 
     if (req->path != NULL) {
+#ifdef PYUV_PYTHON3
+        path = PyUnicode_FromString(req->path);
+#else
         path = PyString_FromString(req->path);
+#endif
+
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -303,15 +333,20 @@ rename_cb(uv_fs_t* req) {
 static void
 chmod_cb(uv_fs_t* req) {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    fs_req_data_t *req_data;
+    PyObject *result, *errorno, *path;
+
     ASSERT(req);
     ASSERT(req->fs_type == UV_FS_CHMOD || req->fs_type == UV_FS_FCHMOD);
 
-    fs_req_data_t *req_data = (fs_req_data_t*)(req->data);
-
-    PyObject *result, *errorno, *path;
+    req_data = (fs_req_data_t*)(req->data);
 
     if (req->path != NULL) {
+#ifdef PYUV_PYTHON3
+        path = PyUnicode_FromString(req->path);
+#else
         path = PyString_FromString(req->path);
+#endif
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -344,15 +379,20 @@ chmod_cb(uv_fs_t* req) {
 static void
 link_cb(uv_fs_t* req) {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    fs_req_data_t *req_data;
+    PyObject *result, *errorno, *path;
+
     ASSERT(req);
     ASSERT(req->fs_type == UV_FS_LINK);
 
-    fs_req_data_t *req_data = (fs_req_data_t*)(req->data);
-
-    PyObject *result, *errorno, *path;
+    req_data = (fs_req_data_t*)(req->data);
 
     if (req->path != NULL) {
+#ifdef PYUV_PYTHON3
+        path = PyUnicode_FromString(req->path);
+#else
         path = PyString_FromString(req->path);
+#endif
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -385,15 +425,20 @@ link_cb(uv_fs_t* req) {
 static void
 symlink_cb(uv_fs_t* req) {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    fs_req_data_t *req_data;
+    PyObject *result, *errorno, *path;
+
     ASSERT(req);
     ASSERT(req->fs_type == UV_FS_SYMLINK);
 
-    fs_req_data_t *req_data = (fs_req_data_t*)(req->data);
-
-    PyObject *result, *errorno, *path;
+    req_data = (fs_req_data_t*)(req->data);
 
     if (req->path != NULL) {
+#ifdef PYUV_PYTHON3
+        path = PyUnicode_FromString(req->path);
+#else
         path = PyString_FromString(req->path);
+#endif
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -426,12 +471,13 @@ symlink_cb(uv_fs_t* req) {
 static void
 readlink_cb(uv_fs_t* req) {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    fs_req_data_t *req_data;
+    PyObject *result, *errorno, *path;
+
     ASSERT(req);
     ASSERT(req->fs_type == UV_FS_READLINK);
 
-    fs_req_data_t *req_data = (fs_req_data_t*)(req->data);
-
-    PyObject *result, *errorno, *path;
+    req_data = (fs_req_data_t*)(req->data);
 
     if (req->result < 0) {
         errorno = PyInt_FromLong((long)req->errorno);
@@ -440,7 +486,11 @@ readlink_cb(uv_fs_t* req) {
     } else {
         errorno = Py_None;
         Py_INCREF(Py_None);
+#ifdef PYUV_PYTHON3
+        path = PyUnicode_FromString(req->ptr);
+#else
         path = PyString_FromString(req->ptr);
+#endif
     }
 
     result = PyObject_CallFunctionObjArgs(req_data->callback, req_data->loop, req_data->data, path, errorno, NULL);
@@ -463,15 +513,20 @@ readlink_cb(uv_fs_t* req) {
 static void
 chown_cb(uv_fs_t* req) {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    fs_req_data_t *req_data;
+    PyObject *result, *errorno, *path;
+
     ASSERT(req);
     ASSERT(req->fs_type == UV_FS_CHOWN || req->fs_type == UV_FS_FCHOWN);
 
-    fs_req_data_t *req_data = (fs_req_data_t*)(req->data);
-
-    PyObject *result, *errorno, *path;
+    req_data = (fs_req_data_t*)(req->data);
 
     if (req->path != NULL) {
+#ifdef PYUV_PYTHON3
+        path = PyUnicode_FromString(req->path);
+#else
         path = PyString_FromString(req->path);
+#endif
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -504,17 +559,21 @@ chown_cb(uv_fs_t* req) {
 static void
 open_cb(uv_fs_t* req) {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    fs_req_data_t *req_data;
+    PyObject *result, *fd, *errorno, *path;
+
     ASSERT(req);
     ASSERT(req->fs_type == UV_FS_OPEN);
 
-    fs_req_data_t *req_data = (fs_req_data_t*)(req->data);
-
-    PyObject *result, *fd, *errorno, *path;
-
+    req_data = (fs_req_data_t*)(req->data);
     fd = PyInt_FromLong((long)req->result);
 
     if (req->path != NULL) {
+#ifdef PYUV_PYTHON3
+        path = PyUnicode_FromString(req->path);
+#else
         path = PyString_FromString(req->path);
+#endif
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -547,15 +606,20 @@ open_cb(uv_fs_t* req) {
 static void
 close_cb(uv_fs_t* req) {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    fs_req_data_t *req_data;
+    PyObject *result, *errorno, *path;
+
     ASSERT(req);
     ASSERT(req->fs_type == UV_FS_CLOSE);
 
-    fs_req_data_t *req_data = (fs_req_data_t*)(req->data);
-
-    PyObject *result, *errorno, *path;
+    req_data = (fs_req_data_t*)(req->data);
 
     if (req->path != NULL) {
+#ifdef PYUV_PYTHON3
+        path = PyUnicode_FromString(req->path);
+#else
         path = PyString_FromString(req->path);
+#endif
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -588,15 +652,20 @@ close_cb(uv_fs_t* req) {
 static void
 read_cb(uv_fs_t* req) {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    fs_req_data_t *req_data;
+    PyObject *result, *errorno, *read_data, *path;
+
     ASSERT(req);
     ASSERT(req->fs_type == UV_FS_READ);
 
-    fs_req_data_t *req_data = (fs_req_data_t*)(req->data);
-
-    PyObject *result, *errorno, *read_data, *path;
+    req_data = (fs_req_data_t*)(req->data);
 
     if (req->path != NULL) {
+#ifdef PYUV_PYTHON3
+        path = PyUnicode_FromString(req->path);
+#else
         path = PyString_FromString(req->path);
+#endif
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -609,7 +678,11 @@ read_cb(uv_fs_t* req) {
     } else {
         errorno = Py_None;
         Py_INCREF(Py_None);
+#ifdef PYUV_PYTHON3
+        read_data = PyUnicode_FromStringAndSize(req_data->buf.base, req->result);
+#else
         read_data = PyString_FromStringAndSize(req_data->buf.base, req->result);
+#endif
     }
 
     result = PyObject_CallFunctionObjArgs(req_data->callback, req_data->loop, req_data->data, path, read_data, errorno, NULL);
@@ -633,17 +706,21 @@ read_cb(uv_fs_t* req) {
 static void
 write_cb(uv_fs_t* req) {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    fs_req_data_t *req_data;
+    PyObject *result, *errorno, *bytes_written, *path;
+
     ASSERT(req);
     ASSERT(req->fs_type == UV_FS_WRITE);
 
-    fs_req_data_t *req_data = (fs_req_data_t*)(req->data);
-
-    PyObject *result, *errorno, *bytes_written, *path;
-
+    req_data = (fs_req_data_t*)(req->data);
     bytes_written = PyInt_FromLong((long)req->result);
 
     if (req->path != NULL) {
+#ifdef PYUV_PYTHON3
+        path = PyUnicode_FromString(req->path);
+#else
         path = PyString_FromString(req->path);
+#endif
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -677,15 +754,20 @@ write_cb(uv_fs_t* req) {
 static void
 fsync_cb(uv_fs_t* req) {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    fs_req_data_t *req_data;
+    PyObject *result, *errorno, *path;
+
     ASSERT(req);
     ASSERT(req->fs_type == UV_FS_FSYNC || req->fs_type == UV_FS_FDATASYNC);
 
-    fs_req_data_t *req_data = (fs_req_data_t*)(req->data);
-
-    PyObject *result, *errorno, *path;
+    req_data = (fs_req_data_t*)(req->data);
 
     if (req->path != NULL) {
+#ifdef PYUV_PYTHON3
+        path = PyUnicode_FromString(req->path);
+#else
         path = PyString_FromString(req->path);
+#endif
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -718,15 +800,20 @@ fsync_cb(uv_fs_t* req) {
 static void
 ftruncate_cb(uv_fs_t* req) {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    fs_req_data_t *req_data;
+    PyObject *result, *errorno, *path;
+
     ASSERT(req);
     ASSERT(req->fs_type == UV_FS_FTRUNCATE);
 
-    fs_req_data_t *req_data = (fs_req_data_t*)(req->data);
-
-    PyObject *result, *errorno, *path;
+    req_data = (fs_req_data_t*)(req->data);
 
     if (req->path != NULL) {
+#ifdef PYUV_PYTHON3
+        path = PyUnicode_FromString(req->path);
+#else
         path = PyString_FromString(req->path);
+#endif
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -759,17 +846,22 @@ ftruncate_cb(uv_fs_t* req) {
 static void
 readdir_cb(uv_fs_t* req) {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    int r;
+    char *ptr;
+    fs_req_data_t *req_data;
+    PyObject *result, *errorno, *files, *item, *path;
+
     ASSERT(req);
     ASSERT(req->fs_type == UV_FS_READDIR);
 
-    fs_req_data_t *req_data = (fs_req_data_t*)(req->data);
-
-    int r;
-    char *ptr;
-    PyObject *result, *errorno, *files, *item, *path;
+    req_data = (fs_req_data_t*)(req->data);
 
     if (req->path != NULL) {
+#ifdef PYUV_PYTHON3
+        path = PyUnicode_FromString(req->path);
+#else
         path = PyString_FromString(req->path);
+#endif
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -792,7 +884,13 @@ readdir_cb(uv_fs_t* req) {
             r = req->result;
             ptr = req->ptr;
             while (r--) {
+#ifdef PYUV_PYTHON3
+
+                item = PyUnicode_FromString(ptr);
+
+#else
                 item = PyString_FromString(ptr);
+#endif
                 PyList_Append(files, item);
                 Py_DECREF(item);
                 ptr += strlen(ptr) + 1;
@@ -820,17 +918,21 @@ readdir_cb(uv_fs_t* req) {
 static void
 sendfile_cb(uv_fs_t* req) {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    fs_req_data_t *req_data;
+    PyObject *result, *errorno, *bytes_written, *path;
+
     ASSERT(req);
     ASSERT(req->fs_type == UV_FS_SENDFILE);
 
-    fs_req_data_t *req_data = (fs_req_data_t*)(req->data);
-
-    PyObject *result, *errorno, *bytes_written, *path;
-
+    req_data = (fs_req_data_t*)(req->data);
     bytes_written = PyInt_FromLong((long)req->result);
 
     if (req->path != NULL) {
+#ifdef PYUV_PYTHON3
+        path = PyUnicode_FromString(req->path);
+#else
         path = PyString_FromString(req->path);
+#endif
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -863,15 +965,20 @@ sendfile_cb(uv_fs_t* req) {
 static void
 utime_cb(uv_fs_t* req) {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    fs_req_data_t *req_data;
+    PyObject *result, *errorno, *path;
+
     ASSERT(req);
     ASSERT(req->fs_type == UV_FS_UTIME || req->fs_type == UV_FS_FUTIME);
 
-    fs_req_data_t *req_data = (fs_req_data_t*)(req->data);
-
-    PyObject *result, *errorno, *path;
+    req_data = (fs_req_data_t*)(req->data);
 
     if (req->path != NULL) {
+#ifdef PYUV_PYTHON3
+        path = PyUnicode_FromString(req->path);
+#else
         path = PyString_FromString(req->path);
+#endif
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -902,15 +1009,15 @@ utime_cb(uv_fs_t* req) {
 
 
 static PyObject *
-stat_func(PyObject *self, PyObject *args, PyObject *kwargs, int type)
+stat_func(PyObject *args, PyObject *kwargs, int type)
 {
     int r;
     char *path;
+    uv_fs_t *fs_req = NULL;
+    fs_req_data_t *req_data = NULL;
     Loop *loop;
     PyObject *callback;
     PyObject *data = Py_None;
-    uv_fs_t *fs_req = NULL;
-    fs_req_data_t *req_data = NULL;
 
     static char *kwlist[] = {"loop", "path", "callback", "data", NULL};
 
@@ -971,32 +1078,35 @@ error:
 
 
 static PyObject *
-FS_func_stat(PyObject *self, PyObject *args, PyObject *kwargs)
+FS_func_stat(PyObject *obj, PyObject *args, PyObject *kwargs)
 {
-    return stat_func(self, args, kwargs, UV_FS_STAT);
+    UNUSED_ARG(obj);
+    return stat_func(args, kwargs, UV_FS_STAT);
 }
 
 
 static PyObject *
-FS_func_lstat(PyObject *self, PyObject *args, PyObject *kwargs)
+FS_func_lstat(PyObject *obj, PyObject *args, PyObject *kwargs)
 {
-    return stat_func(self, args, kwargs, UV_FS_LSTAT);
+    UNUSED_ARG(obj);
+    return stat_func(args, kwargs, UV_FS_LSTAT);
 }
 
 
 static PyObject *
-FS_func_fstat(PyObject *self, PyObject *args, PyObject *kwargs)
+FS_func_fstat(PyObject *obj, PyObject *args, PyObject *kwargs)
 {
-    int r;
-    int fd;
+    int r, fd;
+    uv_fs_t *fs_req = NULL;
+    fs_req_data_t *req_data = NULL;
     Loop *loop;
     PyObject *callback;
     PyObject *data = Py_None;
-    uv_fs_t *fs_req = NULL;
-    fs_req_data_t *req_data = NULL;
 
     static char *kwlist[] = {"loop", "fd", "callback", "data", NULL};
 
+    UNUSED_ARG(obj);
+
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!iO|O:fstat", kwlist, &LoopType, &loop, &fd, &callback, &data)) {
         return NULL;
     }
@@ -1050,18 +1160,20 @@ error:
 
 
 static PyObject *
-FS_func_unlink(PyObject *self, PyObject *args, PyObject *kwargs)
+FS_func_unlink(PyObject *obj, PyObject *args, PyObject *kwargs)
 {
     int r;
     char *path;
+    uv_fs_t *fs_req = NULL;
+    fs_req_data_t *req_data = NULL;
     Loop *loop;
     PyObject *callback;
     PyObject *data = Py_None;
-    uv_fs_t *fs_req = NULL;
-    fs_req_data_t *req_data = NULL;
 
     static char *kwlist[] = {"loop", "path", "callback", "data", NULL};
 
+    UNUSED_ARG(obj);
+
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!sO|O:unlink", kwlist, &LoopType, &loop, &path, &callback, &data)) {
         return NULL;
     }
@@ -1115,19 +1227,20 @@ error:
 
 
 static PyObject *
-FS_func_mkdir(PyObject *self, PyObject *args, PyObject *kwargs)
+FS_func_mkdir(PyObject *obj, PyObject *args, PyObject *kwargs)
 {
-    int r;
-    int mode;
+    int r, mode;
     char *path;
+    uv_fs_t *fs_req = NULL;
+    fs_req_data_t *req_data = NULL;
     Loop *loop;
     PyObject *callback;
     PyObject *data = Py_None;
-    uv_fs_t *fs_req = NULL;
-    fs_req_data_t *req_data = NULL;
 
     static char *kwlist[] = {"loop", "path", "mode", "callback", "data", NULL};
 
+    UNUSED_ARG(obj);
+
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!siO|O:mkdir", kwlist, &LoopType, &loop, &path, &mode, &callback, &data)) {
         return NULL;
     }
@@ -1181,18 +1294,20 @@ error:
 
 
 static PyObject *
-FS_func_rmdir(PyObject *self, PyObject *args, PyObject *kwargs)
+FS_func_rmdir(PyObject *obj, PyObject *args, PyObject *kwargs)
 {
     int r;
     char *path;
+    uv_fs_t *fs_req = NULL;
+    fs_req_data_t *req_data = NULL;
     Loop *loop;
     PyObject *callback;
     PyObject *data = Py_None;
-    uv_fs_t *fs_req = NULL;
-    fs_req_data_t *req_data = NULL;
 
     static char *kwlist[] = {"loop", "path", "callback", "data", NULL};
 
+    UNUSED_ARG(obj);
+
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!sO|O:rmdir", kwlist, &LoopType, &loop, &path, &callback, &data)) {
         return NULL;
     }
@@ -1246,19 +1361,20 @@ error:
 
 
 static PyObject *
-FS_func_rename(PyObject *self, PyObject *args, PyObject *kwargs)
+FS_func_rename(PyObject *obj, PyObject *args, PyObject *kwargs)
 {
     int r;
-    char *path;
-    char *new_path;
+    char *path, *new_path;
+    uv_fs_t *fs_req = NULL;
+    fs_req_data_t *req_data = NULL;
     Loop *loop;
     PyObject *callback;
     PyObject *data = Py_None;
-    uv_fs_t *fs_req = NULL;
-    fs_req_data_t *req_data = NULL;
 
     static char *kwlist[] = {"loop", "path", "new_path", "callback", "data", NULL};
 
+    UNUSED_ARG(obj);
+
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!ssO|O:rename", kwlist, &LoopType, &loop, &path, &new_path, &callback, &data)) {
         return NULL;
     }
@@ -1312,19 +1428,20 @@ error:
 
 
 static PyObject *
-FS_func_chmod(PyObject *self, PyObject *args, PyObject *kwargs)
+FS_func_chmod(PyObject *obj, PyObject *args, PyObject *kwargs)
 {
-    int r;
-    int mode;
+    int r, mode;
     char *path;
+    uv_fs_t *fs_req = NULL;
+    fs_req_data_t *req_data = NULL;
     Loop *loop;
     PyObject *callback;
     PyObject *data = Py_None;
-    uv_fs_t *fs_req = NULL;
-    fs_req_data_t *req_data = NULL;
 
     static char *kwlist[] = {"loop", "path", "mode", "callback", "data", NULL};
 
+    UNUSED_ARG(obj);
+
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!siO|O:chmod", kwlist, &LoopType, &loop, &path, &mode, &callback, &data)) {
         return NULL;
     }
@@ -1378,19 +1495,19 @@ error:
 
 
 static PyObject *
-FS_func_fchmod(PyObject *self, PyObject *args, PyObject *kwargs)
+FS_func_fchmod(PyObject *obj, PyObject *args, PyObject *kwargs)
 {
-    int r;
-    int mode;
-    int fd;
+    int r, mode, fd;
+    uv_fs_t *fs_req = NULL;
+    fs_req_data_t *req_data = NULL;
     Loop *loop;
     PyObject *callback;
     PyObject *data = Py_None;
-    uv_fs_t *fs_req = NULL;
-    fs_req_data_t *req_data = NULL;
 
     static char *kwlist[] = {"loop", "fd", "mode", "callback", "data", NULL};
 
+    UNUSED_ARG(obj);
+
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!iiO|O:fchmod", kwlist, &LoopType, &loop, &fd, &mode, &callback, &data)) {
         return NULL;
     }
@@ -1444,19 +1561,20 @@ error:
 
 
 static PyObject *
-FS_func_link(PyObject *self, PyObject *args, PyObject *kwargs)
+FS_func_link(PyObject *obj, PyObject *args, PyObject *kwargs)
 {
     int r;
-    char *path;
-    char *new_path;
+    char *path, *new_path;
+    uv_fs_t *fs_req = NULL;
+    fs_req_data_t *req_data = NULL;
     Loop *loop;
     PyObject *callback;
     PyObject *data = Py_None;
-    uv_fs_t *fs_req = NULL;
-    fs_req_data_t *req_data = NULL;
 
     static char *kwlist[] = {"loop", "path", "new_path", "callback", "data", NULL};
 
+    UNUSED_ARG(obj);
+
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!ssO|O:link", kwlist, &LoopType, &loop, &path, &new_path, &callback, &data)) {
         return NULL;
     }
@@ -1510,20 +1628,20 @@ error:
 
 
 static PyObject *
-FS_func_symlink(PyObject *self, PyObject *args, PyObject *kwargs)
+FS_func_symlink(PyObject *obj, PyObject *args, PyObject *kwargs)
 {
-    int r;
-    int flags;
-    char *path;
-    char *new_path;
+    int r, flags;
+    char *path, *new_path;
+    uv_fs_t *fs_req = NULL;
+    fs_req_data_t *req_data = NULL;
     Loop *loop;
     PyObject *callback;
     PyObject *data = Py_None;
-    uv_fs_t *fs_req = NULL;
-    fs_req_data_t *req_data = NULL;
 
     static char *kwlist[] = {"loop", "path", "new_path", "flags", "callback", "data", NULL};
 
+    UNUSED_ARG(obj);
+
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!ssiO|O:symlink", kwlist, &LoopType, &loop, &path, &new_path, &flags, &callback, &data)) {
         return NULL;
     }
@@ -1577,18 +1695,20 @@ error:
 
 
 static PyObject *
-FS_func_readlink(PyObject *self, PyObject *args, PyObject *kwargs)
+FS_func_readlink(PyObject *obj, PyObject *args, PyObject *kwargs)
 {
     int r;
     char *path;
+    uv_fs_t *fs_req = NULL;
+    fs_req_data_t *req_data = NULL;
     Loop *loop;
     PyObject *callback;
     PyObject *data = Py_None;
-    uv_fs_t *fs_req = NULL;
-    fs_req_data_t *req_data = NULL;
 
     static char *kwlist[] = {"loop", "path", "callback", "data", NULL};
 
+    UNUSED_ARG(obj);
+
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!sO|O:readlink", kwlist, &LoopType, &loop, &path, &callback, &data)) {
         return NULL;
     }
@@ -1642,20 +1762,20 @@ error:
 
 
 static PyObject *
-FS_func_chown(PyObject *self, PyObject *args, PyObject *kwargs)
+FS_func_chown(PyObject *obj, PyObject *args, PyObject *kwargs)
 {
-    int r;
-    int uid;
-    int gid;
+    int r, uid, gid;
     char *path;
+    uv_fs_t *fs_req = NULL;
+    fs_req_data_t *req_data = NULL;
     Loop *loop;
     PyObject *callback;
     PyObject *data = Py_None;
-    uv_fs_t *fs_req = NULL;
-    fs_req_data_t *req_data = NULL;
 
     static char *kwlist[] = {"loop", "path", "uid", "gid", "callback", "data", NULL};
 
+    UNUSED_ARG(obj);
+
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!siiO|O:chown", kwlist, &LoopType, &loop, &path, &uid, &gid, &callback, &data)) {
         return NULL;
     }
@@ -1709,20 +1829,19 @@ error:
 
 
 static PyObject *
-FS_func_fchown(PyObject *self, PyObject *args, PyObject *kwargs)
+FS_func_fchown(PyObject *obj, PyObject *args, PyObject *kwargs)
 {
-    int r;
-    int fd;
-    int uid;
-    int gid;
+    int r, fd, uid, gid;
+    uv_fs_t *fs_req = NULL;
+    fs_req_data_t *req_data = NULL;
     Loop *loop;
     PyObject *callback;
     PyObject *data = Py_None;
-    uv_fs_t *fs_req = NULL;
-    fs_req_data_t *req_data = NULL;
 
     static char *kwlist[] = {"loop", "fd", "uid", "gid", "callback", "data", NULL};
 
+    UNUSED_ARG(obj);
+
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!iiiO|O:fchown", kwlist, &LoopType, &loop, &fd, &uid, &gid, &callback, &data)) {
         return NULL;
     }
@@ -1776,20 +1895,20 @@ error:
 
 
 static PyObject *
-FS_func_open(PyObject *self, PyObject *args, PyObject *kwargs)
+FS_func_open(PyObject *obj, PyObject *args, PyObject *kwargs)
 {
-    int r;
-    int flags;
-    int mode;
+    int r, flags, mode;
     char *path;
+    uv_fs_t *fs_req = NULL;
+    fs_req_data_t *req_data = NULL;
     Loop *loop;
     PyObject *callback;
     PyObject *data = Py_None;
-    uv_fs_t *fs_req = NULL;
-    fs_req_data_t *req_data = NULL;
 
     static char *kwlist[] = {"loop", "path", "flags", "mode", "callback", "data", NULL};
 
+    UNUSED_ARG(obj);
+
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!siiO|O:open", kwlist, &LoopType, &loop, &path, &flags, &mode, &callback, &data)) {
         return NULL;
     }
@@ -1843,18 +1962,19 @@ error:
 
 
 static PyObject *
-FS_func_close(PyObject *self, PyObject *args, PyObject *kwargs)
+FS_func_close(PyObject *obj, PyObject *args, PyObject *kwargs)
 {
-    int r;
-    int fd;
+    int r, fd;
+    uv_fs_t *fs_req = NULL;
+    fs_req_data_t *req_data = NULL;
     Loop *loop;
     PyObject *callback;
     PyObject *data = Py_None;
-    uv_fs_t *fs_req = NULL;
-    fs_req_data_t *req_data = NULL;
 
     static char *kwlist[] = {"loop", "fd", "callback", "data", NULL};
 
+    UNUSED_ARG(obj);
+
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!iO|O:close", kwlist, &LoopType, &loop, &fd, &callback, &data)) {
         return NULL;
     }
@@ -1908,21 +2028,20 @@ error:
 
 
 static PyObject *
-FS_func_read(PyObject *self, PyObject *args, PyObject *kwargs)
+FS_func_read(PyObject *obj, PyObject *args, PyObject *kwargs)
 {
-    int r;
-    int fd;
-    int length;
-    int offset;
+    int r, fd, length, offset;
     char *buf_data = NULL;
+    uv_fs_t *fs_req = NULL;
+    fs_req_data_t *req_data = NULL;
     Loop *loop;
     PyObject *callback;
     PyObject *data = Py_None;
-    uv_fs_t *fs_req = NULL;
-    fs_req_data_t *req_data = NULL;
 
     static char *kwlist[] = {"loop", "fd", "length", "offset", "callback", "data", NULL};
 
+    UNUSED_ARG(obj);
+
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!iiiO|O:read", kwlist, &LoopType, &loop, &fd, &length, &offset, &callback, &data)) {
         return NULL;
     }
@@ -1987,22 +2106,22 @@ error:
 
 
 static PyObject *
-FS_func_write(PyObject *self, PyObject *args, PyObject *kwargs)
+FS_func_write(PyObject *obj, PyObject *args, PyObject *kwargs)
 {
-    int r;
-    int fd;
-    int offset;
-    char *write_str = NULL;
-    int write_len;
-    char *buf_data = NULL;
+    int r, fd, offset, write_len;
+    char *write_str, *buf_data;
+    uv_fs_t *fs_req = NULL;
+    fs_req_data_t *req_data = NULL;
     Loop *loop;
     PyObject *callback;
     PyObject *data = Py_None;
-    uv_fs_t *fs_req = NULL;
-    fs_req_data_t *req_data = NULL;
 
     static char *kwlist[] = {"loop", "fd", "write_data", "offset", "callback", "data", NULL};
 
+    write_str = buf_data = NULL;
+
+    UNUSED_ARG(obj);
+
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!is#iO|O:write", kwlist, &LoopType, &loop, &fd, &write_str, &write_len, &offset, &callback, &data)) {
         return NULL;
     }
@@ -2067,18 +2186,19 @@ error:
 
 
 static PyObject *
-FS_func_fsync(PyObject *self, PyObject *args, PyObject *kwargs)
+FS_func_fsync(PyObject *obj, PyObject *args, PyObject *kwargs)
 {
-    int r;
-    int fd;
+    int r, fd;
+    uv_fs_t *fs_req = NULL;
+    fs_req_data_t *req_data = NULL;
     Loop *loop;
     PyObject *callback;
     PyObject *data = Py_None;
-    uv_fs_t *fs_req = NULL;
-    fs_req_data_t *req_data = NULL;
 
     static char *kwlist[] = {"loop", "fd", "callback", "data", NULL};
 
+    UNUSED_ARG(obj);
+
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!iO|O:fsync", kwlist, &LoopType, &loop, &fd, &callback, &data)) {
         return NULL;
     }
@@ -2132,18 +2252,19 @@ error:
 
 
 static PyObject *
-FS_func_fdatasync(PyObject *self, PyObject *args, PyObject *kwargs)
+FS_func_fdatasync(PyObject *obj, PyObject *args, PyObject *kwargs)
 {
-    int r;
-    int fd;
+    int r, fd;
+    uv_fs_t *fs_req = NULL;
+    fs_req_data_t *req_data = NULL;
     Loop *loop;
     PyObject *callback;
     PyObject *data = Py_None;
-    uv_fs_t *fs_req = NULL;
-    fs_req_data_t *req_data = NULL;
 
     static char *kwlist[] = {"loop", "fd", "callback", "data", NULL};
 
+    UNUSED_ARG(obj);
+
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!iO|O:fdatasync", kwlist, &LoopType, &loop, &fd, &callback, &data)) {
         return NULL;
     }
@@ -2197,19 +2318,19 @@ error:
 
 
 static PyObject *
-FS_func_ftruncate(PyObject *self, PyObject *args, PyObject *kwargs)
+FS_func_ftruncate(PyObject *obj, PyObject *args, PyObject *kwargs)
 {
-    int r;
-    int fd;
-    int offset;
+    int r, fd, offset;
+    uv_fs_t *fs_req = NULL;
+    fs_req_data_t *req_data = NULL;
     Loop *loop;
     PyObject *callback;
     PyObject *data = Py_None;
-    uv_fs_t *fs_req = NULL;
-    fs_req_data_t *req_data = NULL;
 
     static char *kwlist[] = {"loop", "fd", "offset", "callback", "data", NULL};
 
+    UNUSED_ARG(obj);
+
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!iiO|O:ftruncate", kwlist, &LoopType, &loop, &fd, &offset, &callback, &data)) {
         return NULL;
     }
@@ -2262,21 +2383,22 @@ error:
 }
 
 
-// Aapparently 'flags' doesn't do much (nothing on Windows) because libuv uses ptr from eio, instead of ptr2
+/* Aapparently 'flags' doesn't do much (nothing on Windows) because libuv uses ptr from eio, instead of ptr2 */
 static PyObject *
-FS_func_readdir(PyObject *self, PyObject *args, PyObject *kwargs)
+FS_func_readdir(PyObject *obj, PyObject *args, PyObject *kwargs)
 {
-    int r;
-    int flags;
+    int r, flags;
     char *path;
+    uv_fs_t *fs_req = NULL;
+    fs_req_data_t *req_data = NULL;
     Loop *loop;
     PyObject *callback;
     PyObject *data = Py_None;
-    uv_fs_t *fs_req = NULL;
-    fs_req_data_t *req_data = NULL;
 
     static char *kwlist[] = {"loop", "path", "flags", "callback", "data", NULL};
 
+    UNUSED_ARG(obj);
+
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!siO|O:readdir", kwlist, &LoopType, &loop, &path, &flags, &callback, &data)) {
         return NULL;
     }
@@ -2330,18 +2452,19 @@ error:
 
 
 static PyObject *
-FS_func_sendfile(PyObject *self, PyObject *args, PyObject *kwargs)
+FS_func_sendfile(PyObject *obj, PyObject *args, PyObject *kwargs)
 {
-    int r;
-    int out_fd, in_fd, in_offset, length;
+    int r, out_fd, in_fd, in_offset, length;
+    uv_fs_t *fs_req = NULL;
+    fs_req_data_t *req_data = NULL;
     Loop *loop;
     PyObject *callback;
     PyObject *data = Py_None;
-    uv_fs_t *fs_req = NULL;
-    fs_req_data_t *req_data = NULL;
 
     static char *kwlist[] = {"loop", "out_fd", "in_fd", "in_offset", "length", "callback", "data", NULL};
 
+    UNUSED_ARG(obj);
+
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!iiiiO|O:sendfile", kwlist, &LoopType, &loop, &out_fd, &in_fd, &in_offset, &length, &callback, &data)) {
         return NULL;
     }
@@ -2395,19 +2518,21 @@ error:
 
 
 static PyObject *
-FS_func_utime(PyObject *self, PyObject *args, PyObject *kwargs)
+FS_func_utime(PyObject *obj, PyObject *args, PyObject *kwargs)
 {
     int r;
     double atime, mtime;
     char *path;
+    uv_fs_t *fs_req = NULL;
+    fs_req_data_t *req_data = NULL;
     Loop *loop;
     PyObject *callback;
     PyObject *data = Py_None;
-    uv_fs_t *fs_req = NULL;
-    fs_req_data_t *req_data = NULL;
 
     static char *kwlist[] = {"loop", "path", "atime", "mtime", "callback", "data", NULL};
 
+    UNUSED_ARG(obj);
+
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!sddO|O:utime", kwlist, &LoopType, &loop, &path, &atime, &mtime, &callback, &data)) {
         return NULL;
     }
@@ -2461,19 +2586,20 @@ error:
 
 
 static PyObject *
-FS_func_futime(PyObject *self, PyObject *args, PyObject *kwargs)
+FS_func_futime(PyObject *obj, PyObject *args, PyObject *kwargs)
 {
-    int r;
-    int fd;
+    int r, fd;
     double atime, mtime;
+    uv_fs_t *fs_req = NULL;
+    fs_req_data_t *req_data = NULL;
     Loop *loop;
     PyObject *callback;
     PyObject *data = Py_None;
-    uv_fs_t *fs_req = NULL;
-    fs_req_data_t *req_data = NULL;
 
     static char *kwlist[] = {"loop", "fd", "atime", "mtime", "callback", "data", NULL};
 
+    UNUSED_ARG(obj);
+
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!iddO|O:futime", kwlist, &LoopType, &loop, &fd, &atime, &mtime, &callback, &data)) {
         return NULL;
     }
@@ -2564,12 +2690,13 @@ static void
 on_fsevent_close(uv_handle_t *handle)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
-    ASSERT(handle);
-    FSEvent *self = (FSEvent *)handle->data;
-    ASSERT(self);
-
+    FSEvent *self;
     PyObject *result;
 
+    ASSERT(handle);
+    self = (FSEvent *)handle->data;
+    ASSERT(self);
+
     if (self->on_close_cb != Py_None) {
         result = PyObject_CallFunctionObjArgs(self->on_close_cb, self, NULL);
         if (result == NULL) {
@@ -2603,18 +2730,23 @@ static void
 on_fsevent_callback(uv_fs_event_t *handle, const char *filename, int events, int status)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    FSEvent *self;
+    PyObject *result, *py_filename, *py_events, *errorno;
+
     ASSERT(handle);
     ASSERT(status == 0);
 
-    FSEvent *self = (FSEvent *)handle->data;
+    self = (FSEvent *)handle->data;
     ASSERT(self);
     /* Object could go out of scope in the callback, increase refcount to avoid it */
     Py_INCREF(self);
 
-    PyObject *result, *py_filename, *py_events, *errorno;
-
     if (filename) {
+#ifdef PYUV_PYTHON3
+        py_filename = PyUnicode_FromString(filename);
+#else
         py_filename = PyString_FromString(filename);
+#endif
     } else {
         py_filename = Py_None;
         Py_INCREF(Py_None);
@@ -2644,15 +2776,15 @@ on_fsevent_callback(uv_fs_event_t *handle, const char *filename, int events, int
 static PyObject *
 FSEvent_func_start(FSEvent *self, PyObject *args, PyObject *kwargs)
 {
-    int r = 0;
-    int flags;
+    int r, flags;
     char *path;
-    PyObject *tmp = NULL;
-    PyObject *callback;
     uv_fs_event_t *fs_event = NULL;
+    PyObject *tmp, *callback;
 
     static char *kwlist[] = {"path", "callback", "flags", NULL};
 
+    tmp = NULL;
+
     if (self->uv_handle) {
         PyErr_SetString(PyExc_FSEventError, "FSEvent was already started");
         return NULL;
@@ -2727,8 +2859,10 @@ FSEvent_func_close(FSEvent *self, PyObject *args)
 static int
 FSEvent_tp_init(FSEvent *self, PyObject *args, PyObject *kwargs)
 {
-    PyObject *tmp = NULL;
     Loop *loop;
+    PyObject *tmp = NULL;
+
+    UNUSED_ARG(kwargs);
 
     if (self->uv_handle) {
         PyErr_SetString(PyExc_FSEventError, "Object already initialized");
@@ -2850,12 +2984,25 @@ static PyTypeObject FSEventType = {
     FSEvent_tp_new,                                                 /*tp_new*/
 };
 
+#ifdef PYUV_PYTHON3
+static PyModuleDef pyuv_fs_module = {
+    PyModuleDef_HEAD_INIT,
+    "pyuv.fs",              /*m_name*/
+    NULL,                   /*m_doc*/
+    -1,                     /*m_size*/
+    FS_methods,             /*m_methods*/
+};
+#endif
 
 PyObject *
 init_fs(void)
 {
     PyObject *module;
+#ifdef PYUV_PYTHON3
+    module = PyModule_Create(&pyuv_fs_module);
+#else
     module = Py_InitModule("pyuv.fs", FS_methods);
+#endif
     if (module == NULL) {
         return NULL;
     }
