@@ -6,16 +6,17 @@ static void
 on_pipe_connection(uv_stream_t* server, int status)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    Pipe *self;
+    IOStream *base;
+    PyObject *result, *py_errorno;
     ASSERT(server);
 
-    Pipe *self = (Pipe *)server->data;
+    self = (Pipe *)server->data;
     ASSERT(self);
     /* Object could go out of scope in the callback, increase refcount to avoid it */
     Py_INCREF(self);
 
-    IOStream *base = (IOStream *)self;
-
-    PyObject *result, *py_errorno;
+    base = (IOStream *)self;
 
     if (status != 0) {
         uv_err_t err = uv_last_error(UV_LOOP(base));
@@ -40,20 +41,21 @@ static void
 on_pipe_client_connection(uv_connect_t *req, int status)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    iostream_req_data_t* req_data;
+    Pipe *self;
+    IOStream *base;
+    PyObject *callback, *result, *py_errorno;
     ASSERT(req);
 
-    iostream_req_data_t* req_data = (iostream_req_data_t *)req->data;
-
-    Pipe *self = (Pipe *)req_data->obj;
-    PyObject *callback = req_data->callback;
-
-    PyObject *result, *py_errorno;
+    req_data = (iostream_req_data_t *)req->data;
+    self = (Pipe *)req_data->obj;
+    callback = req_data->callback;
 
     ASSERT(self);
     /* Object could go out of scope in the callback, increase refcount to avoid it */
     Py_INCREF(self);
 
-    IOStream *base = (IOStream *)self;
+    base = (IOStream *)self;
 
     if (status != 0) {
         uv_err_t err = uv_last_error(UV_LOOP(base));
@@ -71,7 +73,6 @@ on_pipe_client_connection(uv_connect_t *req, int status)
 
     Py_DECREF(callback);
     PyMem_Free(req_data);
-    req->data = NULL;
     PyMem_Free(req);
 
     Py_DECREF(self);
@@ -83,15 +84,16 @@ static void
 on_pipe_read2(uv_pipe_t* handle, int nread, uv_buf_t buf, uv_handle_type pending)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    uv_err_t err;
+    IOStream *self;
+    PyObject *result, *data, *py_errorno, *py_pending;
     ASSERT(handle);
 
-    IOStream *self = (IOStream *)handle->data;
+    self = (IOStream *)handle->data;
     ASSERT(self);
     /* Object could go out of scope in the callback, increase refcount to avoid it */
     Py_INCREF(self);
 
-    PyObject *result, *data, *py_errorno, *py_pending;
-
     py_pending = PyInt_FromLong((long)pending);
 
     if (nread >= 0) {
@@ -101,7 +103,7 @@ on_pipe_read2(uv_pipe_t* handle, int nread, uv_buf_t buf, uv_handle_type pending
     } else if (nread < 0) {
         data = Py_None;
         Py_INCREF(Py_None);
-        uv_err_t err = uv_last_error(UV_LOOP(self));
+        err = uv_last_error(UV_LOOP(self));
         py_errorno = PyInt_FromLong((long)err.code);
     }
 
@@ -121,7 +123,7 @@ on_pipe_read2(uv_pipe_t* handle, int nread, uv_buf_t buf, uv_handle_type pending
 static PyObject *
 Pipe_func_bind(Pipe *self, PyObject *args)
 {
-    int r = 0;
+    int r;
     char *name;
 
     IOStream *base = (IOStream *)self;
@@ -148,13 +150,14 @@ Pipe_func_bind(Pipe *self, PyObject *args)
 static PyObject *
 Pipe_func_listen(Pipe *self, PyObject *args)
 {
-    int r;
-    int backlog = 128;
-    PyObject *callback;
-    PyObject *tmp = NULL;
+    int r, backlog;
+    PyObject *callback, *tmp;
 
     IOStream *base = (IOStream *)self;
 
+    backlog = 128;
+    tmp = NULL;
+
     if (!base->uv_handle) {
         PyErr_SetString(PyExc_PipeError, "already closed");
         return NULL;
@@ -192,7 +195,7 @@ Pipe_func_listen(Pipe *self, PyObject *args)
 static PyObject *
 Pipe_func_accept(TCP *self, PyObject *args)
 {
-    int r = 0;
+    int r;
     PyObject *client;
 
     IOStream *base = (IOStream *)self;
@@ -222,7 +225,7 @@ Pipe_func_accept(TCP *self, PyObject *args)
 
 
 static PyObject *
-Pipe_func_connect(Pipe *self, PyObject *args, PyObject *kwargs)
+Pipe_func_connect(Pipe *self, PyObject *args)
 {
     char *name;
     uv_connect_t *connect_req = NULL;
@@ -268,13 +271,10 @@ Pipe_func_connect(Pipe *self, PyObject *args, PyObject *kwargs)
 
 error:
     if (connect_req) {
-        connect_req->data = NULL;
         PyMem_Free(connect_req);
     }
     if (req_data) {
         Py_DECREF(callback);
-        req_data->obj = NULL;
-        req_data->callback = NULL;
         PyMem_Free(req_data);
     }
     return NULL;
@@ -329,12 +329,13 @@ Pipe_func_pending_instances(Pipe *self, PyObject *args)
 static PyObject *
 Pipe_func_start_read2(Pipe *self, PyObject *args)
 {
-    int r = 0;
-    PyObject *tmp = NULL;
-    PyObject *callback;
+    int r;
+    PyObject *tmp, *callback;
 
     IOStream *base = (IOStream *)self;
 
+    tmp = NULL;
+
     if (!base->uv_handle) {
         PyErr_SetString(PyExc_PipeError, "Pipe is closed");
         return NULL;
@@ -367,15 +368,11 @@ Pipe_func_start_read2(Pipe *self, PyObject *args)
 static PyObject *
 Pipe_func_write2(Pipe *self, PyObject *args)
 {
-    int i, n;
-    int r = 0;
-    int buf_count = 0;
+    int i, n, r, buf_count;
     char *data_str;
     char *tmp;
     Py_ssize_t data_len;
-    PyObject *item;
-    PyObject *data;
-    PyObject *callback = Py_None;
+    PyObject *item, *data, *callback;
     TCP *send_handle;
     uv_buf_t tmpbuf;
     uv_buf_t *bufs = NULL;
@@ -385,6 +382,9 @@ Pipe_func_write2(Pipe *self, PyObject *args)
 
     IOStream *base = (IOStream *)self;
 
+    buf_count = 0;
+    callback = Py_None;
+
     if (!base->uv_handle) {
         PyErr_SetString(PyExc_PipeError, "Pipe is closed");
         return NULL;
@@ -428,7 +428,7 @@ Pipe_func_write2(Pipe *self, PyObject *args)
     wr->data = (void *)req_data;
 
     if (PyString_Check(data)) {
-        // We have a single string
+        /* We have a single string */
         bufs = (uv_buf_t *) PyMem_Malloc(sizeof(uv_buf_t));
         if (!bufs) {
             PyErr_NoMemory();
@@ -447,7 +447,7 @@ Pipe_func_write2(Pipe *self, PyObject *args)
         bufs[0] = tmpbuf;
         buf_count = 1;
     } else {
-        // We have a list
+        /* We have a list */
         buf_count = 0;
         n = PySequence_Length(data);
         bufs = (uv_buf_t *) PyMem_Malloc(sizeof(uv_buf_t) * n);
@@ -496,13 +496,9 @@ error:
     }
     if (req_data) {
         Py_DECREF(callback);
-        req_data->obj = NULL;
-        req_data->callback = NULL;
-        req_data->data = NULL;
         PyMem_Free(req_data);
     }
     if (wr) {
-        wr->data = NULL;
         PyMem_Free(wr);
     }
     return NULL;
@@ -512,14 +508,16 @@ error:
 static int
 Pipe_tp_init(Pipe *self, PyObject *args, PyObject *kwargs)
 {
-    int r = 0;
+    int r;
+    uv_pipe_t *uv_stream;
     Loop *loop;
     PyObject *tmp = NULL;
     PyObject *ipc = Py_False;
-    uv_pipe_t *uv_stream;
 
     IOStream *base = (IOStream *)self;
 
+    UNUSED_ARG(kwargs);
+
     if (base->uv_handle) {
         PyErr_SetString(PyExc_PipeError, "Object already initialized");
         return -1;
