@@ -20,12 +20,13 @@ static void
 on_udp_close(uv_handle_t *handle)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
-    ASSERT(handle);
-    UDP *self = (UDP *)handle->data;
-    ASSERT(self);
-
+    UDP *self;
     PyObject *result;
 
+    ASSERT(handle);
+    self = (UDP *)handle->data;
+    ASSERT(self);
+
     if (self->on_close_cb != Py_None) {
         result = PyObject_CallFunctionObjArgs(self->on_close_cb, self, NULL);
         if (result == NULL) {
@@ -59,8 +60,10 @@ static uv_buf_t
 on_udp_alloc(uv_udp_t* handle, size_t suggested_size)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    uv_buf_t buf;
     ASSERT(suggested_size <= UDP_MAX_BUF_SIZE);
-    uv_buf_t buf = uv_buf_init(PyMem_Malloc(suggested_size), suggested_size);
+    buf = uv_buf_init(PyMem_Malloc(suggested_size), suggested_size);
+    UNUSED_ARG(handle);
     PyGILState_Release(gstate);
     return buf;
 }
@@ -70,20 +73,19 @@ static void
 on_udp_read(uv_udp_t* handle, int nread, uv_buf_t buf, struct sockaddr* addr, unsigned flags)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
-
-    char ip4[INET_ADDRSTRLEN];
-    char ip6[INET6_ADDRSTRLEN];
-    int r = 0;
+    char ip[INET6_ADDRSTRLEN];
     struct sockaddr_in addr4;
     struct sockaddr_in6 addr6;
-
+    uv_err_t err;
+    UDP *self;
     PyObject *result, *address_tuple, *data, *py_errorno;
 
     ASSERT(handle);
     ASSERT(flags == 0);
 
-    UDP *self = (UDP *)handle->data;
+    self = (UDP *)handle->data;
     ASSERT(self);
+
     /* Object could go out of scope in the callback, increase refcount to avoid it */
     Py_INCREF(self);
 
@@ -91,14 +93,12 @@ on_udp_read(uv_udp_t* handle, int nread, uv_buf_t buf, struct sockaddr* addr, un
         ASSERT(addr);
         if (addr->sa_family == AF_INET) {
             addr4 = *(struct sockaddr_in*)addr;
-            r = uv_ip4_name(&addr4, ip4, INET_ADDRSTRLEN);
-            ASSERT(r == 0);
-            address_tuple = Py_BuildValue("(si)", ip4, ntohs(addr4.sin_port));
+            uv_ip4_name(&addr4, ip, INET_ADDRSTRLEN);
+            address_tuple = Py_BuildValue("(si)", ip, ntohs(addr4.sin_port));
         } else {
             addr6 = *(struct sockaddr_in6*)addr;
-            r = uv_ip6_name(&addr6, ip6, INET6_ADDRSTRLEN);
-            ASSERT(r == 0);
-            address_tuple = Py_BuildValue("(si)", ip6, ntohs(addr6.sin6_port));
+            uv_ip6_name(&addr6, ip, INET6_ADDRSTRLEN);
+            address_tuple = Py_BuildValue("(si)", ip, ntohs(addr6.sin6_port));
         }
         data = PyString_FromStringAndSize(buf.base, nread);
         py_errorno = Py_None;
@@ -108,7 +108,7 @@ on_udp_read(uv_udp_t* handle, int nread, uv_buf_t buf, struct sockaddr* addr, un
         Py_INCREF(Py_None);
         data = Py_None;
         Py_INCREF(Py_None);
-        uv_err_t err = uv_last_error(UV_LOOP(self));
+        err = uv_last_error(UV_LOOP(self));
         py_errorno = PyInt_FromLong((long)err.code);
     }
 
@@ -130,23 +130,25 @@ on_udp_read(uv_udp_t* handle, int nread, uv_buf_t buf, struct sockaddr* addr, un
 static void
 on_udp_send(uv_udp_send_t* req, int status)
 {
-    int i;
-
     PyGILState_STATE gstate = PyGILState_Ensure();
+    int i;
+    udp_req_data_t* req_data;
+    udp_send_data_t* send_data;
+    UDP *self;
+    PyObject *callback, *result, *py_errorno;
+
     ASSERT(req);
 
-    udp_req_data_t* req_data = (udp_req_data_t *)req->data;
-    udp_send_data_t* send_data = (udp_send_data_t *)req_data->data;
+    req_data = (udp_req_data_t *)req->data;
+    send_data = (udp_send_data_t *)req_data->data;
 
-    UDP *self = (UDP *)req_data->obj;
-    PyObject *callback = req_data->callback;
+    self = (UDP *)req_data->obj;
+    callback = req_data->callback;
 
     ASSERT(self);
     /* Object could go out of scope in the callback, increase refcount to avoid it */
     Py_INCREF(self);
 
-    PyObject *result, *py_errorno;
-
     if (callback != Py_None) {
         if (status < 0) {
             uv_err_t err = uv_last_error(UV_LOOP(self));
@@ -168,7 +170,6 @@ on_udp_send(uv_udp_send_t* req, int status)
     PyMem_Free(send_data);
     Py_DECREF(callback);
     PyMem_Free(req_data);
-    req->data = NULL;
     PyMem_Free(req);
 
     Py_DECREF(self);
@@ -179,10 +180,8 @@ on_udp_send(uv_udp_send_t* req, int status)
 static PyObject *
 UDP_func_bind(UDP *self, PyObject *args)
 {
-    int r = 0;
+    int r, bind_port, address_type;
     char *bind_ip;
-    int bind_port;
-    int address_type;
     struct in_addr addr4;
     struct in6_addr addr6;
 
@@ -200,9 +199,9 @@ UDP_func_bind(UDP *self, PyObject *args)
         return NULL;
     }
 
-    if (inet_pton(AF_INET, bind_ip, &addr4) == 1) {
+    if (uv_inet_pton(AF_INET, bind_ip, &addr4) == 1) {
         address_type = AF_INET;
-    } else if (inet_pton(AF_INET6, bind_ip, &addr6) == 1) {
+    } else if (uv_inet_pton(AF_INET6, bind_ip, &addr6) == 1) {
         address_type = AF_INET6;
     } else {
         PyErr_SetString(PyExc_ValueError, "invalid IP address");
@@ -227,9 +226,10 @@ UDP_func_bind(UDP *self, PyObject *args)
 static PyObject *
 UDP_func_start_recv(UDP *self, PyObject *args)
 {
-    int r = 0;
-    PyObject *tmp = NULL;
-    PyObject *callback;
+    int r;
+    PyObject *tmp, *callback;
+
+    tmp = NULL;
 
     if (!self->uv_handle) {
         PyErr_SetString(PyExc_UDPError, "closed");
@@ -263,7 +263,7 @@ UDP_func_start_recv(UDP *self, PyObject *args)
 static PyObject *
 UDP_func_stop_recv(UDP *self)
 {
-    int r = 0;
+    int r;
 
     if (!self->uv_handle) {
         PyErr_SetString(PyExc_UDPError, "closed");
@@ -282,15 +282,8 @@ UDP_func_stop_recv(UDP *self)
 static PyObject *
 UDP_func_send(UDP *self, PyObject *args)
 {
-    int i, n;
-    int r = 0;
-    int buf_count = 0;
-    char *data_str;
-    char *tmp;
-    Py_ssize_t data_len;
-    char *dest_ip;
-    int dest_port;
-    int address_type;
+    int i, n, r, buf_count, dest_port, address_type;
+    char *data_str, *tmp, *dest_ip;
     struct in_addr addr4;
     struct in6_addr addr6;
     uv_buf_t tmpbuf;
@@ -298,10 +291,11 @@ UDP_func_send(UDP *self, PyObject *args)
     uv_udp_send_t *wr = NULL;
     udp_req_data_t *req_data = NULL;
     udp_send_data_t *send_data = NULL;
-    PyObject *item;
-    PyObject *data;
-    PyObject *address_tuple;
-    PyObject *callback = Py_None;
+    Py_ssize_t data_len;
+    PyObject *item, *data, *address_tuple, *callback;
+
+    buf_count = 0;
+    callback = Py_None;
 
     if (!self->uv_handle) {
         PyErr_SetString(PyExc_UDPError, "closed");
@@ -331,9 +325,9 @@ UDP_func_send(UDP *self, PyObject *args)
         return NULL;
     }
 
-    if (inet_pton(AF_INET, dest_ip, &addr4) == 1) {
+    if (uv_inet_pton(AF_INET, dest_ip, &addr4) == 1) {
         address_type = AF_INET;
-    } else if (inet_pton(AF_INET6, dest_ip, &addr6) == 1) {
+    } else if (uv_inet_pton(AF_INET6, dest_ip, &addr6) == 1) {
         address_type = AF_INET6;
     } else {
         PyErr_SetString(PyExc_ValueError, "invalid IP address");
@@ -364,7 +358,7 @@ UDP_func_send(UDP *self, PyObject *args)
     wr->data = (void *)req_data;
 
     if (PyString_Check(data)) {
-        // We have a single string
+        /* We have a single string */
         bufs = (uv_buf_t *) PyMem_Malloc(sizeof(uv_buf_t));
         if (!bufs) {
             PyErr_NoMemory();
@@ -383,7 +377,7 @@ UDP_func_send(UDP *self, PyObject *args)
         bufs[0] = tmpbuf;
         buf_count = 1;
     } else {
-        // We have a list
+        /* We have a list */
         buf_count = 0;
         n = PySequence_Length(data);
         bufs = (uv_buf_t *) PyMem_Malloc(sizeof(uv_buf_t) * n);
@@ -429,13 +423,9 @@ error:
     }
     if (req_data) {
         Py_DECREF(callback);
-        req_data->obj = NULL;
-        req_data->callback = NULL;
-        req_data->data = NULL;
         PyMem_Free(req_data);
     }
     if (wr) {
-        wr->data = NULL;
         PyMem_Free(wr);
     }
     return NULL;
@@ -445,17 +435,17 @@ error:
 static PyObject *
 UDP_func_set_membership(UDP *self, PyObject *args)
 {
-    int r = 0;
-    char *multicast_address;
-    char *interface_address = NULL;
-    int membership;
+    int r, membership;
+    char *multicast_address, *interface_address;
+
+    interface_address = NULL;
 
     if (!self->uv_handle) {
         PyErr_SetString(PyExc_UDPError, "closed");
         return NULL;
     }
 
-    if (!PyArg_ParseTuple(args, "si|:set_membership", &multicast_address, &membership, &interface_address)) {
+    if (!PyArg_ParseTuple(args, "si|s:set_membership", &multicast_address, &membership, &interface_address)) {
         return NULL;
     }
 
@@ -504,13 +494,13 @@ UDP_func_close(UDP *self, PyObject *args)
 static PyObject *
 UDP_func_getsockname(UDP *self)
 {
-    int r = 0;
+    int r, namelen;
+    char ip[INET6_ADDRSTRLEN];
     struct sockaddr sockname;
     struct sockaddr_in *addr4;
     struct sockaddr_in6 *addr6;
-    char ip4[INET_ADDRSTRLEN];
-    char ip6[INET6_ADDRSTRLEN];
-    int namelen = sizeof(sockname);
+
+    namelen = sizeof(sockname);
 
     if (!self->uv_handle) {
         PyErr_SetString(PyExc_UDPError, "closed");
@@ -525,14 +515,12 @@ UDP_func_getsockname(UDP *self)
 
     if (sockname.sa_family == AF_INET) {
         addr4 = (struct sockaddr_in*)&sockname;
-        r = uv_ip4_name(addr4, ip4, INET_ADDRSTRLEN);
-        ASSERT(r == 0);
-        return Py_BuildValue("si", ip4, ntohs(addr4->sin_port));
+        uv_ip4_name(addr4, ip, INET_ADDRSTRLEN);
+        return Py_BuildValue("si", ip, ntohs(addr4->sin_port));
     } else if (sockname.sa_family == AF_INET6) {
         addr6 = (struct sockaddr_in6*)&sockname;
-        r = uv_ip6_name(addr6, ip6, INET6_ADDRSTRLEN);
-        ASSERT(r == 0);
-        return Py_BuildValue("si", ip6, ntohs(addr6->sin6_port));
+        uv_ip6_name(addr6, ip, INET6_ADDRSTRLEN);
+        return Py_BuildValue("si", ip, ntohs(addr6->sin6_port));
     } else {
         PyErr_SetString(PyExc_UDPError, "unknown address type detected");
         return NULL;
@@ -543,10 +531,12 @@ UDP_func_getsockname(UDP *self)
 static int
 UDP_tp_init(UDP *self, PyObject *args, PyObject *kwargs)
 {
-    int r = 0;
+    int r;
+    uv_udp_t *uv_udp_handle = NULL;
     Loop *loop;
     PyObject *tmp = NULL;
-    uv_udp_t *uv_udp_handle = NULL;
+
+    UNUSED_ARG(kwargs);
 
     if (self->uv_handle) {
         PyErr_SetString(PyExc_UDPError, "Object already initialized");
