@@ -2,9 +2,12 @@
 static PyObject* PyExc_UVError;
 
 static PyObject *
-Util_func_hrtime(PyObject *self)
+Util_func_hrtime(PyObject *obj)
 {
     PyObject *val;
+
+    UNUSED_ARG(obj);
+
 #if SIZEOF_TIME_T > SIZEOF_LONG
     val = PyLong_FromLongLong((PY_LONG_LONG)uv_hrtime());
 #else
@@ -15,9 +18,12 @@ Util_func_hrtime(PyObject *self)
 
 
 static PyObject *
-Util_func_get_free_memory(PyObject *self)
+Util_func_get_free_memory(PyObject *obj)
 {
     PyObject *val;
+
+    UNUSED_ARG(obj);
+
 #if SIZEOF_TIME_T > SIZEOF_LONG
     val = PyLong_FromLongLong((PY_LONG_LONG)uv_get_free_memory());
 #else
@@ -28,9 +34,12 @@ Util_func_get_free_memory(PyObject *self)
 
 
 static PyObject *
-Util_func_get_total_memory(PyObject *self)
+Util_func_get_total_memory(PyObject *obj)
 {
     PyObject *val;
+
+    UNUSED_ARG(obj);
+
 #if SIZEOF_TIME_T > SIZEOF_LONG
     val = PyLong_FromLongLong((PY_LONG_LONG)uv_get_total_memory());
 #else
@@ -41,25 +50,31 @@ Util_func_get_total_memory(PyObject *self)
 
 
 static PyObject *
-Util_func_loadavg(PyObject *self)
+Util_func_loadavg(PyObject *obj)
 {
     double avg[3];
+
+    UNUSED_ARG(obj);
+
     uv_loadavg(avg);
     return Py_BuildValue("(ddd)", avg[0], avg[1], avg[2]);
 }
 
 
 static PyObject *
-Util_func_uptime(PyObject *self)
+Util_func_uptime(PyObject *obj)
 {
     double uptime;
     uv_err_t err;
+    PyObject *exc_data;
+
+    UNUSED_ARG(obj);
 
     err = uv_uptime(&uptime);
     if (err.code == UV_OK) {
         return PyFloat_FromDouble(uptime);
     } else {
-        PyObject *exc_data = Py_BuildValue("(is)", err.code, uv_strerror(err));
+        exc_data = Py_BuildValue("(is)", err.code, uv_strerror(err));
         if (exc_data != NULL) {
             PyErr_SetObject(PyExc_UVError, exc_data);
             Py_DECREF(exc_data);
@@ -70,16 +85,19 @@ Util_func_uptime(PyObject *self)
 
 
 static PyObject *
-Util_func_resident_set_memory(PyObject *self)
+Util_func_resident_set_memory(PyObject *obj)
 {
     size_t rss;
     uv_err_t err;
+    PyObject *exc_data;
+
+    UNUSED_ARG(obj);
 
     err = uv_resident_set_memory(&rss);
     if (err.code == UV_OK) {
         return PyInt_FromSsize_t(rss);
     } else {
-        PyObject *exc_data = Py_BuildValue("(is)", err.code, uv_strerror(err));
+        exc_data = Py_BuildValue("(is)", err.code, uv_strerror(err));
         if (exc_data != NULL) {
             PyErr_SetObject(PyExc_UVError, exc_data);
             Py_DECREF(exc_data);
@@ -90,15 +108,15 @@ Util_func_resident_set_memory(PyObject *self)
 
 
 static PyObject *
-Util_func_interface_addresses(PyObject *self)
+Util_func_interface_addresses(PyObject *obj)
 {
-    int i;
-    int count;
-    char ip4[INET_ADDRSTRLEN];
-    char ip6[INET6_ADDRSTRLEN];
+    int i, count;
+    char ip[INET6_ADDRSTRLEN];
     uv_interface_address_t* interfaces;
     uv_err_t err;
-    PyObject *result, *item;
+    PyObject *result, *item, *exc_data;
+
+    UNUSED_ARG(obj);
 
     err = uv_interface_addresses(&interfaces, &count);
     if (err.code == UV_OK) {
@@ -115,11 +133,11 @@ Util_func_interface_addresses(PyObject *self)
             PyDict_SetItemString(item, "name", PyString_FromString(interfaces[i].name));
             PyDict_SetItemString(item, "is_internal", PyBool_FromLong((long)interfaces[i].is_internal));
             if (interfaces[i].address.address4.sin_family == AF_INET) {
-                uv_ip4_name(&interfaces[i].address.address4, ip4, INET_ADDRSTRLEN);
-                PyDict_SetItemString(item, "address", PyString_FromString(ip4));
+                uv_ip4_name(&interfaces[i].address.address4, ip, INET_ADDRSTRLEN);
+                PyDict_SetItemString(item, "address", PyString_FromString(ip));
             } else if (interfaces[i].address.address4.sin_family == AF_INET6) {
-                uv_ip6_name(&interfaces[i].address.address6, ip6, INET6_ADDRSTRLEN);
-                PyDict_SetItemString(item, "address", PyString_FromString(ip6));
+                uv_ip6_name(&interfaces[i].address.address6, ip, INET6_ADDRSTRLEN);
+                PyDict_SetItemString(item, "address", PyString_FromString(ip));
             }
             if (PyList_Append(result, item))
                 continue;
@@ -128,7 +146,7 @@ Util_func_interface_addresses(PyObject *self)
         uv_free_interface_addresses(interfaces, count);
         return result;
     } else {
-        PyObject *exc_data = Py_BuildValue("(is)", err.code, uv_strerror(err));
+        exc_data = Py_BuildValue("(is)", err.code, uv_strerror(err));
         if (exc_data != NULL) {
             PyErr_SetObject(PyExc_UVError, exc_data);
             Py_DECREF(exc_data);
@@ -151,13 +169,14 @@ set_cpu_info_time_val(PyObject *dict, char *key, uint64_t v)
 }
 
 static PyObject *
-Util_func_cpu_info(PyObject *self)
+Util_func_cpu_info(PyObject *obj)
 {
-    int i;
-    int count;
+    int i, count;
     uv_cpu_info_t* cpus;
     uv_err_t err;
-    PyObject *result, *item, *times;
+    PyObject *result, *item, *times, *exc_data;
+
+    UNUSED_ARG(obj);
 
     err = uv_cpu_info(&cpus, &count);
     if (err.code == UV_OK) {
@@ -187,7 +206,7 @@ Util_func_cpu_info(PyObject *self)
         uv_free_cpu_info(cpus, count);
         return result;
     } else {
-        PyObject *exc_data = Py_BuildValue("(is)", err.code, uv_strerror(err));
+        exc_data = Py_BuildValue("(is)", err.code, uv_strerror(err));
         if (exc_data != NULL) {
             PyErr_SetObject(PyExc_UVError, exc_data);
             Py_DECREF(exc_data);
@@ -210,12 +229,25 @@ Util_methods[] = {
     { NULL }
 };
 
+#ifdef PYUV_PYTHON3
+static PyModuleDef pyuv_util_module = {
+    PyModuleDef_HEAD_INIT,
+    "pyuv.util",            /*m_name*/
+    NULL,                   /*m_doc*/
+    -1,                     /*m_size*/
+    Util_methods,           /*m_methods*/
+};
+#endif
 
 PyObject *
 init_util(void)
 {
     PyObject *module;
+#ifdef PYUV_PYTHON3
+    module = PyModule_Create(&pyuv_util_module);
+#else
     module = Py_InitModule("pyuv.util", Util_methods);
+#endif
     if (module == NULL) {
         return NULL;
     }
