@@ -3,7 +3,7 @@ import os
 import shutil
 import stat
 
-import common
+from common import platform_skip, unittest2
 import pyuv
 
 
@@ -15,17 +15,16 @@ TEST_DIR = 'test-dir'
 TEST_DIR2 = 'test-dir_2'
 BAD_DIR = 'test-dir-bad'
 
-class FSTestStat(common.UVTestCase):
+
+class FSTestStat(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
         with open(TEST_FILE, 'w') as f:
             f.write('test')
-        os.symlink(TEST_FILE, TEST_LINK)
 
     def tearDown(self):
         os.remove(TEST_FILE)
-        os.remove(TEST_LINK)
 
     def stat_cb(self, loop, data, path, stat_result, errorno):
         self.errorno = errorno
@@ -42,17 +41,37 @@ class FSTestStat(common.UVTestCase):
         self.loop.run()
         self.assertEqual(self.errorno, None)
 
-    def lstat_cb(self, loop, data, path, stat_result, errorno):
+
+@platform_skip(["win32"])
+class FSTestLstat(unittest2.TestCase):
+
+    def setUp(self):
+        self.loop = pyuv.Loop.default_loop()
+        with open(TEST_FILE, 'w') as f:
+            f.write('test')
+        os.symlink(TEST_FILE, TEST_LINK)
+
+    def tearDown(self):
+        os.remove(TEST_FILE)
+        os.remove(TEST_LINK)
+
+    def stat_cb(self, loop, data, path, stat_result, errorno):
         self.errorno = errorno
 
-    def test_lstat(self):
+    def test_stat(self):
         self.errorno = None
-        pyuv.fs.lstat(self.loop, TEST_LINK, self.lstat_cb)
+        pyuv.fs.stat(self.loop, TEST_FILE, self.stat_cb)
         self.loop.run()
         self.assertEqual(self.errorno, None)
 
+    def test_lstat(self):
+        self.errorno = None
+        pyuv.fs.lstat(self.loop, TEST_LINK, self.stat_cb)
+        self.loop.run()
+        self.assertEqual(self.errorno, None)
 
-class FSTestFstat(common.UVTestCase):
+@platform_skip(["win32"])
+class FSTestFstat(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
@@ -73,7 +92,7 @@ class FSTestFstat(common.UVTestCase):
         self.assertEqual(self.errorno, None)
 
 
-class FSTestUnlink(common.UVTestCase):
+class FSTestUnlink(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
@@ -105,11 +124,11 @@ class FSTestUnlink(common.UVTestCase):
         self.assertEqual(self.errorno, None)
 
 
-class FSTestMkdir(common.UVTestCase):
+class FSTestMkdir(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
-        os.mkdir(BAD_DIR, 0755)
+        os.mkdir(BAD_DIR, 0o755)
 
     def tearDown(self):
         os.rmdir(BAD_DIR)
@@ -123,23 +142,23 @@ class FSTestMkdir(common.UVTestCase):
 
     def test_bad_mkdir(self):
         self.errorno = None
-        pyuv.fs.mkdir(self.loop, BAD_DIR, 0755, self.mkdir_cb)
+        pyuv.fs.mkdir(self.loop, BAD_DIR, 0o755, self.mkdir_cb)
         self.loop.run()
         self.assertEqual(self.errorno, pyuv.errno.UV_EEXIST)
 
     def test_mkdir(self):
         self.errorno = None
-        pyuv.fs.mkdir(self.loop, TEST_DIR, 0755, self.mkdir_cb)
+        pyuv.fs.mkdir(self.loop, TEST_DIR, 0o755, self.mkdir_cb)
         self.loop.run()
         self.assertEqual(self.errorno, None)
         self.assertTrue(os.path.isdir(TEST_DIR))
 
 
-class FSTestRmdir(common.UVTestCase):
+class FSTestRmdir(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
-        os.mkdir(TEST_DIR, 0755)
+        os.mkdir(TEST_DIR, 0o755)
 
     def tearDown(self):
         try:
@@ -164,7 +183,7 @@ class FSTestRmdir(common.UVTestCase):
         self.assertFalse(os.path.isdir(TEST_DIR))
 
 
-class FSTestRename(common.UVTestCase):
+class FSTestRename(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
@@ -193,7 +212,7 @@ class FSTestRename(common.UVTestCase):
         self.assertTrue(os.path.exists(TEST_FILE2))
 
 
-class FSTestChmod(common.UVTestCase):
+class FSTestChmod(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
@@ -208,14 +227,15 @@ class FSTestChmod(common.UVTestCase):
 
     def test_chmod(self):
         self.errorno = None
-        pyuv.fs.chmod(self.loop, TEST_FILE, 0777, self.chmod_cb)
+        pyuv.fs.chmod(self.loop, TEST_FILE, 0o777, self.chmod_cb)
         self.loop.run()
         self.assertEqual(self.errorno, None)
         mode = os.stat(TEST_FILE).st_mode
         self.assertTrue(bool(mode & stat.S_IRWXU) and bool(mode & stat.S_IRWXG) and bool(mode & stat.S_IRWXO))
 
 
-class FSTestFchmod(common.UVTestCase):
+@platform_skip(["win32"])
+class FSTestFchmod(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
@@ -231,14 +251,14 @@ class FSTestFchmod(common.UVTestCase):
 
     def test_fchmod(self):
         self.errorno = None
-        pyuv.fs.fchmod(self.loop, self.file.fileno(), 0777, self.fchmod_cb)
+        pyuv.fs.fchmod(self.loop, self.file.fileno(), 0o777, self.fchmod_cb)
         self.loop.run()
         self.assertEqual(self.errorno, None)
         mode = os.stat(TEST_FILE).st_mode
         self.assertTrue(bool(mode & stat.S_IRWXU) and bool(mode & stat.S_IRWXG) and bool(mode & stat.S_IRWXO))
 
 
-class FSTestLink(common.UVTestCase):
+class FSTestLink(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
@@ -260,7 +280,8 @@ class FSTestLink(common.UVTestCase):
         self.assertEqual(os.stat(TEST_FILE).st_ino, os.stat(TEST_LINK).st_ino)
 
 
-class FSTestSymlink(common.UVTestCase):
+@platform_skip(["win32"])
+class FSTestSymlink(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
@@ -269,7 +290,10 @@ class FSTestSymlink(common.UVTestCase):
 
     def tearDown(self):
         os.remove(TEST_FILE)
-        os.remove(TEST_LINK)
+        try:
+            os.remove(TEST_LINK)
+        except OSError:
+            pass
 
     def symlink_cb(self, loop, data, path, errorno):
         self.errorno = errorno
@@ -282,7 +306,8 @@ class FSTestSymlink(common.UVTestCase):
         self.assertTrue(os.path.islink(TEST_LINK))
 
 
-class FSTestReadlink(common.UVTestCase):
+@platform_skip(["win32"])
+class FSTestReadlink(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
@@ -307,7 +332,7 @@ class FSTestReadlink(common.UVTestCase):
         self.assertEqual(self.link_path, TEST_FILE)
 
 
-class FSTestChown(common.UVTestCase):
+class FSTestChown(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
@@ -327,7 +352,7 @@ class FSTestChown(common.UVTestCase):
         self.assertEqual(self.errorno, None)
 
 
-class FSTestFchown(common.UVTestCase):
+class FSTestFchown(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
@@ -348,7 +373,7 @@ class FSTestFchown(common.UVTestCase):
         self.assertEqual(self.errorno, None)
 
 
-class FSTestOpen(common.UVTestCase):
+class FSTestOpen(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
@@ -386,7 +411,8 @@ class FSTestOpen(common.UVTestCase):
         self.assertEqual(self.errorno, pyuv.errno.UV_ENOENT)
 
 
-class FSTestRead(common.UVTestCase):
+@platform_skip(["win32"])
+class FSTestRead(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
@@ -411,7 +437,8 @@ class FSTestRead(common.UVTestCase):
         self.assertEqual(self.data, 'test')
 
 
-class FSTestWrite(common.UVTestCase):
+@platform_skip(["win32"])
+class FSTestWrite(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
@@ -443,7 +470,9 @@ class FSTestWrite(common.UVTestCase):
         self.assertEqual(self.errorno, None)
         self.assertEqual(open(TEST_FILE, 'r').read(), "TES\x00T")
 
-class FSTestFsync(common.UVTestCase):
+
+@platform_skip(["win32"])
+class FSTestFsync(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
@@ -471,7 +500,8 @@ class FSTestFsync(common.UVTestCase):
         self.assertEqual(open(TEST_FILE, 'r').read(), "TEST")
 
 
-class FSTestFtruncate(common.UVTestCase):
+@platform_skip(["win32"])
+class FSTestFtruncate(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
@@ -501,12 +531,12 @@ class FSTestFtruncate(common.UVTestCase):
         self.assertEqual(open(TEST_FILE, 'r').read(), "")
 
 
-class FSTestReaddir(common.UVTestCase):
+class FSTestReaddir(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
-        os.mkdir(TEST_DIR, 0755)
-        os.mkdir(os.path.join(TEST_DIR, TEST_DIR2), 0755)
+        os.mkdir(TEST_DIR, 0o755)
+        os.mkdir(os.path.join(TEST_DIR, TEST_DIR2), 0o755)
         with open(os.path.join(TEST_DIR, TEST_FILE), 'w') as f:
             f.write('test')
         with open(os.path.join(TEST_DIR, TEST_FILE2), 'w') as f:
@@ -537,7 +567,8 @@ class FSTestReaddir(common.UVTestCase):
         self.assertTrue(TEST_DIR2 in self.files)
 
 
-class FSTestSendfile(common.UVTestCase):
+@platform_skip(["win32"])
+class FSTestSendfile(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
@@ -569,7 +600,8 @@ class FSTestSendfile(common.UVTestCase):
         self.assertEqual(open(TEST_FILE, 'r').read(), open(TEST_FILE2, 'r').read())
 
 
-class FSTestUtime(common.UVTestCase):
+@platform_skip(["win32"])
+class FSTestUtime(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
@@ -591,7 +623,8 @@ class FSTestUtime(common.UVTestCase):
         self.loop.run()
         self.assertEqual(self.errorno, None)
         s = os.stat(TEST_FILE)
-        self.assertTrue(s.st_atime == atime and s.st_mtime == mtime)
+        self.assertEqual(s.st_atime, atime)
+        self.assertEqual(s.st_mtime, mtime)
 
     def test_futime(self):
         self.errorno = None
@@ -603,8 +636,8 @@ class FSTestUtime(common.UVTestCase):
         self.assertTrue(s.st_atime == atime and s.st_mtime == mtime)
 
 
-class FSEventTestBasic(common.UVTestCase):
-    __disabled__ = ["linux"]
+@platform_skip(["linux", "darwin"])
+class FSEventTestBasic(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
@@ -645,11 +678,12 @@ class FSEventTestBasic(common.UVTestCase):
         self.assertTrue(self.events & pyuv.fs.UV_RENAME)
 
 
-class FSEventTest(common.UVTestCase):
+@platform_skip(["darwin"])
+class FSEventTest(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
-        os.mkdir(TEST_DIR, 0755)
+        os.mkdir(TEST_DIR, 0o755)
         with open(os.path.join(TEST_DIR, TEST_FILE), 'w') as f:
             f.write("test")
         with open(TEST_FILE, 'w') as f:
@@ -667,7 +701,11 @@ class FSEventTest(common.UVTestCase):
             pass
 
     def on_fsevent_cb(self, handle, filename, events, errorno):
-        handle.close()
+        try:
+            handle.close()
+        except pyuv.error.FSEventError:
+            # This shouldn't happen, need to verify with libuv
+            return
         self.filename = filename
         self.events = events
         self.errorno = errorno
@@ -708,7 +746,5 @@ class FSEventTest(common.UVTestCase):
 
 
 if __name__ == '__main__':
-    import unittest
-    tests = unittest.TestSuite(common.suites)
-    unittest.TextTestRunner(verbosity=2).run(tests)
+    unittest2.main(verbosity=2)
 
