@@ -2,11 +2,11 @@
 import threading
 import time
 
-import common
+from common import unittest2
 import pyuv
 
 
-class ThreadPoolTest(common.UVTestCase):
+class ThreadPoolTest(unittest2.TestCase):
 
     def setUp(self):
         self.pool_cb_called = 0
@@ -24,7 +24,7 @@ class ThreadPoolTest(common.UVTestCase):
         self.assertEqual(self.pool_cb_called, 3)
 
 
-class ThreadPoolMultiLoopTest(common.UVTestCase):
+class ThreadPoolMultiLoopTest(unittest2.TestCase):
 
     def setUp(self):
         self.pool_cb_called = 0
@@ -50,7 +50,5 @@ class ThreadPoolMultiLoopTest(common.UVTestCase):
 
 
 if __name__ == '__main__':
-    import unittest
-    tests = unittest.TestSuite(common.suites)
-    unittest.TextTestRunner(verbosity=2).run(tests)
+    unittest2.main(verbosity=2)
 
