@@ -1,6 +1,8 @@
 
 import os
+import sys
 
+from common import unittest2, platform_skip
 import common
 import pyuv
 
@@ -9,7 +11,7 @@ TEST_PORT = 12345
 TEST_PORT2 = 12346
 MULTICAST_ADDRESS = "239.255.0.1"
 
-class UDPTest(common.UVTestCase):
+class UDPTest(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
@@ -19,19 +21,21 @@ class UDPTest(common.UVTestCase):
     def on_close(self, handle):
         self.on_close_called += 1
 
-    def on_server_recv(self, handle, (ip, port), data, error):
+    def on_server_recv(self, handle, ip_port, data, error):
+        ip, port = ip_port
         data = data.strip()
-        self.assertEquals(data, "PING")
-        self.server.send("PONG"+os.linesep, (ip, port))
+        self.assertEquals(data, b"PING")
+        self.server.send(b"PONG"+common.linesep, (ip, port))
 
-    def on_client_recv(self, handle, (ip, port), data, error):
+    def on_client_recv(self, handle, ip_port, data, error):
+        ip, port = ip_port
         data = data.strip()
-        self.assertEquals(data, "PONG")
+        self.assertEquals(data, b"PONG")
         self.client.close(self.on_close)
         self.server.close(self.on_close)
 
     def timer_cb(self, timer):
-        self.client.send("PING"+os.linesep, ("127.0.0.1", TEST_PORT))
+        self.client.send(b"PING"+common.linesep, ("127.0.0.1", TEST_PORT))
         timer.close(self.on_close)
 
     def test_udp_pingpong(self):
@@ -48,7 +52,7 @@ class UDPTest(common.UVTestCase):
         self.assertEqual(self.on_close_called, 3)
 
 
-class UDPTestNull(common.UVTestCase):
+class UDPTestNull(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
@@ -58,19 +62,21 @@ class UDPTestNull(common.UVTestCase):
     def on_close(self, handle):
         self.on_close_called += 1
 
-    def on_server_recv(self, handle, (ip, port), data, error):
+    def on_server_recv(self, handle, ip_port, data, error):
+        ip, port = ip_port
         data = data.strip()
-        self.assertEquals(data, "PIN\x00G")
-        self.server.send("PONG"+os.linesep, (ip, port))
+        self.assertEquals(data, b"PIN\x00G")
+        self.server.send(b"PONG"+common.linesep, (ip, port))
 
-    def on_client_recv(self, handle, (ip, port), data, error):
+    def on_client_recv(self, handle, ip_port, data, error):
+        ip, port = ip_port
         data = data.strip()
-        self.assertEquals(data, "PONG")
+        self.assertEquals(data, b"PONG")
         self.client.close(self.on_close)
         self.server.close(self.on_close)
 
     def timer_cb(self, timer):
-        self.client.send("PIN\x00G"+os.linesep, ("127.0.0.1", TEST_PORT))
+        self.client.send(b"PIN\x00G"+common.linesep, ("127.0.0.1", TEST_PORT))
         timer.close(self.on_close)
 
     def test_udp_pingpong_null(self):
@@ -87,7 +93,7 @@ class UDPTestNull(common.UVTestCase):
         self.assertEqual(self.on_close_called, 3)
 
 
-class UDPTestList(common.UVTestCase):
+class UDPTestList(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
@@ -97,19 +103,21 @@ class UDPTestList(common.UVTestCase):
     def on_close(self, handle):
         self.on_close_called += 1
 
-    def on_server_recv(self, handle, (ip, port), data, error):
+    def on_server_recv(self, handle, ip_port, data, error):
+        ip, port = ip_port
         data = data.strip()
-        self.assertEquals(data, "PING")
-        self.server.send(["PONG", os.linesep], (ip, port))
+        self.assertEquals(data, b"PING")
+        self.server.send([b"PONG", common.linesep], (ip, port))
 
-    def on_client_recv(self, handle, (ip, port), data, error):
+    def on_client_recv(self, handle, ip_port, data, error):
+        ip, port = ip_port
         data = data.strip()
-        self.assertEquals(data, "PONG")
+        self.assertEquals(data, b"PONG")
         self.client.close(self.on_close)
         self.server.close(self.on_close)
 
     def timer_cb(self, timer):
-        self.client.send(["PING", os.linesep], ("127.0.0.1", TEST_PORT))
+        self.client.send([b"PING", common.linesep], ("127.0.0.1", TEST_PORT))
         timer.close(self.on_close)
 
     def test_udp_pingpong_list(self):
@@ -126,7 +134,7 @@ class UDPTestList(common.UVTestCase):
         self.assertEqual(self.on_close_called, 3)
 
 
-class UDPTestListNull(common.UVTestCase):
+class UDPTestListNull(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
@@ -136,19 +144,21 @@ class UDPTestListNull(common.UVTestCase):
     def on_close(self, handle):
         self.on_close_called += 1
 
-    def on_server_recv(self, handle, (ip, port), data, error):
+    def on_server_recv(self, handle, ip_port, data, error):
+        ip, port = ip_port
         data = data.strip()
-        self.assertEquals(data, "PIN\x00G")
-        self.server.send(["PONG", os.linesep], (ip, port))
+        self.assertEquals(data, b"PIN\x00G")
+        self.server.send([b"PONG", common.linesep], (ip, port))
 
-    def on_client_recv(self, handle, (ip, port), data, error):
+    def on_client_recv(self, handle, ip_port, data, error):
+        ip, port = ip_port
         data = data.strip()
-        self.assertEquals(data, "PONG")
+        self.assertEquals(data, b"PONG")
         self.client.close(self.on_close)
         self.server.close(self.on_close)
 
     def timer_cb(self, timer):
-        self.client.send(["PIN\x00G", os.linesep], ("127.0.0.1", TEST_PORT))
+        self.client.send([b"PIN\x00G", common.linesep], ("127.0.0.1", TEST_PORT))
         timer.close(self.on_close)
 
     def test_udp_pingpong_list_null(self):
@@ -165,7 +175,7 @@ class UDPTestListNull(common.UVTestCase):
         self.assertEqual(self.on_close_called, 3)
 
 
-class UDPTestInvalidData(common.UVTestCase):
+class UDPTestInvalidData(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
@@ -175,13 +185,18 @@ class UDPTestInvalidData(common.UVTestCase):
     def on_close(self, handle):
         self.on_close_called += 1
 
-    def on_server_recv(self, handle, (ip, port), data):
+    def on_server_recv(self, handle, ip_port, data):
+        ip, port = ip_port
         self.client.close(self.on_close)
         self.server.close(self.on_close)
         self.fail("Expected send to fail.")
 
     def timer_cb(self, timer):
-        self.assertRaises(TypeError, self.client.send, u'Unicode' + os.linesep, ("127.0.0.1", TEST_PORT))
+        if sys.version_info >= (3, 0):
+            data = 'Unicode'
+        else:
+            data = unicode('Unicode')
+        self.assertRaises(TypeError, self.client.send, data+os.linesep, ("127.0.0.1", TEST_PORT))
         self.assertRaises(TypeError, self.client.send, 1, ("127.0.0.1", TEST_PORT))
 
         self.client.close(self.on_close)
@@ -201,7 +216,8 @@ class UDPTestInvalidData(common.UVTestCase):
         self.assertEqual(self.on_close_called, 3)
 
 
-class UDPTestMulticast(common.UVTestCase):
+@platform_skip(["win32"])
+class UDPTestMulticast(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
@@ -212,7 +228,8 @@ class UDPTestMulticast(common.UVTestCase):
     def on_close(self, handle):
         self.on_close_called += 1
 
-    def on_client_recv(self, handle, (ip, port), data, error):
+    def on_client_recv(self, handle, ip_port, data, error):
+        ip, port = ip_port
         self.received_data = data.strip()
         self.client.set_membership(MULTICAST_ADDRESS, pyuv.UV_LEAVE_GROUP)
         self.client.close(self.on_close)
@@ -227,14 +244,12 @@ class UDPTestMulticast(common.UVTestCase):
         self.client.bind((MULTICAST_ADDRESS, TEST_PORT))
         self.client.set_membership(MULTICAST_ADDRESS, pyuv.UV_JOIN_GROUP)
         self.client.start_recv(self.on_client_recv)
-        self.server.send("PING", (MULTICAST_ADDRESS, TEST_PORT), self.on_server_send)
+        self.server.send(b"PING", (MULTICAST_ADDRESS, TEST_PORT), self.on_server_send)
         self.loop.run()
         self.assertEqual(self.on_close_called, 2)
-        self.assertEquals(self.received_data, "PING")
+        self.assertEquals(self.received_data, b"PING")
 
 
 if __name__ == '__main__':
-    import unittest
-    tests = unittest.TestSuite(common.suites)
-    unittest.TextTestRunner(verbosity=2).run(tests)
+    unittest2.main(verbosity=2)
 
